/*  RAxML-VI-HPC (version 2.2) a program for sequential and parallel estimation of phylogenetic trees 
 *  Copyright August 2006 by Alexandros Stamatakis
 *
 *  Partially derived from
 *  fastDNAml, a program for estimation of phylogenetic trees from sequences by Gary J. Olsen
 *  
 *  and 
 *
 *  Programs of the PHYLIP package by Joe Felsenstein.
 *
 *  This program is free software; you may redistribute it and/or modify its
 *  under the terms of the GNU General Public License as published by the Free
 *  Software Foundation; either version 2 of the License, or (at your option)
 *  any later version.
 *
 *  This program is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *  or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 *  for more details.
 * 
 *
 *  For any other enquiries send an Email to Alexandros Stamatakis
 *  Alexandros.Stamatakis@epfl.ch
 *
 *  When publishing work that is based on the results from RAxML-VI-HPC please cite:
 *
 *  Alexandros Stamatakis:"RAxML-VI-HPC: maximum likelihood-based phylogenetic analyses with thousands of taxa and mixed models". 
 *  Bioinformatics 2006; doi: 10.1093/bioinformatics/btl446
 */

#ifndef WIN32
#include <sys/times.h>
#include <sys/types.h>
#include <sys/time.h>
#include <unistd.h> 
#endif

#include <math.h>
#include <time.h> 
#include <stdlib.h>
#include <stdio.h>
#include <ctype.h>
#include <string.h>


#include "axml.h"

extern int optimizeRatesInvocations;
extern int optimizeRateCategoryInvocations;
extern int optimizeAlphaInvocations;
extern int optimizeTTRatioInvocations;
extern int optimizeInvarInvocations;

extern boolean (*newview)           (tree *, nodeptr);
extern double  (*evaluate)          (tree *, nodeptr);   
extern double  (*makenewz)          (tree *, nodeptr, nodeptr, double, int);
extern void    (*evaluateVector)    (tree *, nodeptr, double *);
#ifdef EVAL_PARTIAL_ITERATIVE
extern double  (*evaluatePartial)   (tree *, nodeptr, int, double, nodeptr *, int);
#else
extern double  (*evaluatePartial)   (tree *, nodeptr, int, double);
#endif

extern double  (*evaluatePartition) (tree *, nodeptr, int);
extern boolean (*newviewPartition)  (tree *, nodeptr, int);
extern double  (*makenewzPartition) (tree *t, nodeptr, nodeptr, double, int, int);
extern void (*categorizeGAMMA) (tree *, nodeptr);



void makeVal(char code, double *val)
{
  double i_value;
  int j;

  assert(code >= 0 && code <= 22);

  if(code < 22)
    i_value = 0.0;
  else
    i_value = 1.0;	 

  for(j = 0; j < 20; j++)
    val[j] = i_value;
  
  if(code < 20)
    val[code] = 1.0;
  else
    {
      if(code == 20)	  
	val[2] = val[3] = 1.0;
      if(code == 21)
	val[5] = val[6] = 1.0;	  		   
    }
}


void baseFrequenciesGTR(rawdata *rdta, cruncheddata *cdta, tree *tr, analdef *adef)
{
  double  sum, suma, sumc, sumg, sumt, wj, fa, fc, fg, ft, freqa, freqc, freqg, freqt, pfreqs[20], sumf[20], val[20], temp[20], acc;
  int     i, j, k, l, code;
  char  *yptr;

  if(!adef->useMultipleModel)
    {
      if(adef->model == M_PROTCAT || adef->model == M_PROTGAMMA)
	{	 	  	 	  
	  for(l = 0; l < 20; l++)	    
	    pfreqs[l] = 0.05;	     	    

	  for (k = 1; k <= 8; k++) 
	    {	     	   	    	      			
	      for(l = 0; l < 20; l++)
		sumf[l] = 0.0;

	      for (i = 0; i < rdta->numsp; i++) 
		{		 
		  yptr = rdta->y[i];
		  for(j = 0; j < cdta->endsite; j++) 
		    {		      		     		    		     		      
		      makeVal(yptr[j], val);		     				     				     
		      for(l = 0; l < 20; l++)						
			temp[l] = pfreqs[l] * val[l];
			  						  
		      acc = 0;
		      for(l = 0; l < 20; l++)
			{
			  if(temp[l] != 0.0)
			    acc += temp[l];
			}		      		      
		      
		      wj = cdta->aliaswgt[j] / acc;
		      
		      for(l = 0; l < 20; l++)
			{
			  if(temp[l] != 0.0)
			    {
			      sumf[l] += wj * temp[l];			     				   			     
			    }
			}
		    }
		}	    
	  
	      acc = 0;
	      for(l = 0; l < 20; l++)
		acc += sumf[l];
	      for(l = 0; l < 20; l++)
		pfreqs[l] = sumf[l] / acc;	     
	    }

	  {
	    int countZeros = 0;	       
	    
	    for(l = 0; l < 20; l++)
	      {
		if(pfreqs[l] == 0.0)
		  countZeros++;
	      }		
	    
	    if(countZeros > 0)
	      {
		double correction;
		correction =  (FREQ_MIN  * (double)countZeros)/ ((double)(20 - countZeros)); 
		
		for(l = 0; l < 20; l++)
		  {
		    if(pfreqs[l] == 0.0)
		      pfreqs[l] = FREQ_MIN;
		    else
		      pfreqs[l] -= correction;
		    
		    tr->frequencies[l] = pfreqs[l];		       
		  }
	      }
	    else
	      {
		for(l = 0; l < 20; l++)		      		  
		  tr->frequencies[l] = pfreqs[l];		       		      
	      }	    
	  }	  
	}
      else
	{	 
	  freqa = 0.25;
	  freqc = 0.25;
	  freqg = 0.25;
	  freqt = 0.25;
	  	  

	  for (k = 1; k <= 8; k++) 
	    {
	      suma = 0.0;
	      sumc = 0.0;
	      sumg = 0.0;
	      sumt = 0.0;
	      
	      for (i = 0; i < rdta->numsp; i++) 
		{
		  yptr = rdta->y[i];
		  for (j = 0; j < cdta->endsite; j++) 
		    {
		      code = *yptr++;
		      
		      if(code > 0)
			{
			  fa = freqa * (double)( code       & 1);
			  fc = freqc * (double)((code >> 1) & 1);
			  fg = freqg * (double)((code >> 2) & 1);
			  ft = freqt * (double)((code >> 3) & 1);
			  wj = ((double)cdta->aliaswgt[j]) / (fa + fc + fg + ft);
			  suma += wj * fa;
			  sumc += wj * fc;
			  sumg += wj * fg;
			  sumt += wj * ft;
			}
		    }
		}
	      sum = suma + sumc + sumg + sumt;
	      freqa = suma / sum;
	      freqc = sumc / sum;
	      freqg = sumg / sum;
	      freqt = sumt / sum;
	    }
      
	  if(freqa == 0.0 || freqc == 0.0 || freqg == 0.0 || freqt == 0.0)
	    {
	      printf("ERROR: one of the DNA base freqs is equal to zero\n");
	      exit(-1);
	    }	 

	  tr->frequencies[0] = freqa;
	  tr->frequencies[1] = freqc;
	  tr->frequencies[2] = freqg;
	  tr->frequencies[3] = freqt;
	}
    }
  else
    {
      int model;
      int lower;
      int upper;

      for(model = 0; model < tr->NumberOfModels; model++)
	{
	  lower = tr->modelIndices[model][0];
	  upper = tr->modelIndices[model][1];	  	 

	  if(adef->model == M_PROTCAT || adef->model == M_PROTGAMMA)
	    {	 	  	 	  
	      for(l = 0; l < 20; l++)	    
		pfreqs[l] = 0.05;	     	    
	      
	      for (k = 1; k <= 8; k++) 
		{	     	   	    	      			
		  for(l = 0; l < 20; l++)
		    sumf[l] = 0.0;
		  
		  for (i = 0; i < rdta->numsp; i++) 
		    {		 
		      yptr = rdta->y[i];
		      for(j = lower; j < upper; j++) 
			{		      		     		    		     		      
			  makeVal(yptr[j], val);		     				     				     
			  for(l = 0; l < 20; l++)
			    {
			      if(val[l] != 0.0)
				temp[l] = pfreqs[l] * val[l];
			      else
				temp[l] = 0.0;
			    }
			  
			  acc = 0;
			  for(l = 0; l < 20; l++)
			    {
			      if(temp[l] != 0.0)
				acc += temp[l];
			    }
			  			  			  
			  wj = ((double)cdta->aliaswgt[j]) / acc;
			  
			  for(l = 0; l < 20; l++)
			    {
			      if(temp[l] != 0.0)
				{
				  sumf[l] += wj * temp[l];			     				   			     
				}
			    }
			}
		    }	    
		  
		  acc = 0;
		  for(l = 0; l < 20; l++)
		    {
		      if(sumf[l] != 0.0)
			acc += sumf[l];
		    }
		  
		  for(l = 0; l < 20; l++)
		    pfreqs[l] = sumf[l] / acc;	     
		}

	      {
		int countZeros = 0;	       

		for(l = 0; l < 20; l++)
		  {
		    if(pfreqs[l] == 0.0)
		      countZeros++;
		  }		

		if(countZeros > 0)
		  {
		    double correction;
		    correction =  (FREQ_MIN  * (double)countZeros)/ ((double)(20 - countZeros)); 

		    for(l = 0; l < 20; l++)
		      {
			if(pfreqs[l] == 0.0)
			  pfreqs[l] = FREQ_MIN;
			else
			  pfreqs[l] -= correction;
		    
			tr->frequencies[model * 20 + l] = pfreqs[l];		       
		      }
		  }
		else
		  {
		    for(l = 0; l < 20; l++)		      
		      tr->frequencies[model * 20 + l] = pfreqs[l];		       		      
		  }
	  			
	      } 
	    }
	  else
	    {

	      freqa = 0.25;
	      freqc = 0.25;
	      freqg = 0.25;
	      freqt = 0.25;
	      for (k = 1; k <= 8; k++) 
		{
		  suma = 0.0;
		  sumc = 0.0;
		  sumg = 0.0;
		  sumt = 0.0;
		  for (i = 0; i < rdta->numsp; i++) 
		    {
		      yptr = &(rdta->y[i][lower]);
		      for (j = lower; j < upper; j++) 
			{
			  code = *yptr++;
			  if(code > 0)
			    {
			      fa = freqa * ( code       & 1);
			      fc = freqc * ((code >> 1) & 1);
			      fg = freqg * ((code >> 2) & 1);
			      ft = freqt * ((code >> 3) & 1);
			      wj = cdta->aliaswgt[j] / (fa + fc + fg + ft);
			      suma += wj * fa;
			      sumc += wj * fc;
			      sumg += wj * fg;
			      sumt += wj * ft;
			    }
			}
		    }
		  sum = suma + sumc + sumg + sumt;
		  freqa = suma / sum;
		  freqc = sumc / sum;
		  freqg = sumg / sum;
		  freqt = sumt / sum;
		}
	      
	      if(freqa == 0.0 || freqc == 0.0 || freqg == 0.0 || freqt == 0.0)
		{
		  printf("ERROR: one of the DNA base freqs is equal to zero\n");
		  exit(-1);
		}

	      tr->frequencies[model * 4] = freqa;
	      tr->frequencies[model * 4 + 1] = freqc;
	      tr->frequencies[model * 4 + 2] = freqg;
	      tr->frequencies[model * 4 + 3] = freqt;
	    }
	  
	}      
    }
  return;
}


void initProtMat(tree *tr, int model, double f[20], int proteinMatrix)
{ 
  double q[20][20];
  double daa[400], max, temp;
  int i, j, r;
  double *initialRates = &(tr->initialRates[model * tr->numberOfRates]);
  double scaler;

  switch(proteinMatrix)
    {


    case DAYHOFF/* Ex178 */:
      {	
	daa [ 1*20+ 0] =  	0.0000000000 	;
	daa [ 2*20+ 0] =  	0.3452122600 	; daa [ 2*20+ 1] =  	0.1094938200 	;
	daa [ 3*20+ 0] =  	0.4743301300 	; daa [ 3*20+ 1] =  	0.0274117610 	; daa [ 3*20+ 2] =  	15.4367280000 	;
	daa [ 4*20+ 0] = 	3.5337595000 	; daa [ 4*20+ 1] = 	1.9299909000 	; daa [ 4*20+ 2] = 	0.8112866300 	; daa [ 4*20+ 3] = 	0.2005388000 	;
	daa [ 5*20+ 0] = 	0.2105421200 	; daa [ 5*20+ 1] = 	4.9339183000 	; daa [ 5*20+ 2] = 	0.8577021900 	; daa [ 5*20+ 3] = 	0.2593304800 	; daa [ 5*20+ 4] = 	0.0000000000 	;
	daa [ 6*20+ 0] = 	0.9147679400 	; daa [ 6*20+ 1] = 	0.0000000000 	; daa [ 6*20+ 2] = 	0.6459291500 	; daa [ 6*20+ 3] = 	8.8826696000 	; daa [ 6*20+ 4] = 	0.4082525500 	; daa [ 6*20+ 5] = 	3.7989292000 	;
	daa [ 7*20+ 0] = 	3.4948368000 	; daa [ 7*20+ 1] = 	0.6082377900 	; daa [ 7*20+ 2] = 	2.7739095000 	; daa [ 7*20+ 3] = 	2.3491272000 	; daa [ 7*20+ 4] = 	1.1687220000 	; daa [ 7*20+ 5] = 	0.0000000000 	; daa [ 7*20+ 6] = 	2.6158242000 	;
	daa [ 8*20+ 0] = 	0.0000000000 	; daa [ 8*20+ 1] = 	2.9382568000 	; daa [ 8*20+ 2] = 	5.8913437000 	; daa [ 8*20+ 3] = 	2.1291430000 	; daa [ 8*20+ 4] = 	6.4268487000 	; daa [ 8*20+ 5] = 	10.6558950000 	; daa [ 8*20+ 6] = 	0.1350229300 	; daa [ 8*20+ 7] = 	0.0000000000 	;
	daa [ 9*20+ 0] = 	0.5335693300 	; daa [ 9*20+ 1] = 	0.0000000000 	; daa [ 9*20+ 2] = 	0.5444293300 	; daa [ 9*20+ 3] = 	0.0000000000 	; daa [ 9*20+ 4] = 	0.4523017300 	; daa [ 9*20+ 5] = 	0.0000000000 	; daa [ 9*20+ 6] = 	0.0000000000 	; daa [ 9*20+ 7] = 	0.0000000000 	; daa [ 9*20+ 8] = 	0.1530471100 	;
	daa [10*20+ 0] = 	0.3332529800 	; daa [10*20+ 1] = 	0.4127241100 	; daa [10*20+ 2] = 	0.0215958600 	; daa [10*20+ 3] = 	0.0000000000 	; daa [10*20+ 4] = 	0.6060316100 	; daa [10*20+ 5] = 	1.3976506000 	; daa [10*20+ 6] = 	0.0263782470 	; daa [10*20+ 7] = 	0.0690095570 	; daa [10*20+ 8] = 	0.9941893500 	; daa [10*20+ 9] = 	10.6224430000 	;
	daa [11*20+ 0] = 	0.1270318100 	; daa [11*20+ 1] = 	0.7244565200 	; daa [11*20+ 2] = 	5.6463167000 	; daa [11*20+ 3] = 	0.0072095718 	; daa [11*20+ 4] = 	0.0000000000 	; daa [11*20+ 5] = 	4.7926235000 	; daa [11*20+ 6] = 	3.7962206000 	; daa [11*20+ 7] = 	0.3728100300 	; daa [11*20+ 8] = 	0.3392552200 	; daa [11*20+ 9] = 	0.0000000000 	; daa [11*20+10] = 	0.0367874750 	;
	daa [12*20+ 0] = 	3.9771471000 	; daa [12*20+ 1] = 	0.0000000000 	; daa [12*20+ 2] = 	0.0000000000 	; daa [12*20+ 3] = 	0.0000000000 	; daa [12*20+ 4] = 	0.0000000000 	; daa [12*20+ 5] = 	0.5725830700 	; daa [12*20+ 6] = 	0.1371141900 	; daa [12*20+ 7] = 	0.1893597500 	; daa [12*20+ 8] = 	0.1078445600 	; daa [12*20+ 9] = 	22.8706020000 	; daa [12*20+10] = 	21.2279660000 	; daa [12*20+11] = 	2.1360688000 	;
	daa [13*20+ 0] = 	0.4717790000 	; daa [13*20+ 1] = 	0.0000000000 	; daa [13*20+ 2] = 	0.0000000000 	; daa [13*20+ 3] = 	0.0000000000 	; daa [13*20+ 4] = 	9.8630783000 	; daa [13*20+ 5] = 	0.0543870690 	; daa [13*20+ 6] = 	0.0216289460 	; daa [13*20+ 7] = 	0.1165822400 	; daa [13*20+ 8] = 	1.7877515000 	; daa [13*20+ 9] = 	2.2608881000 	; daa [13*20+10] = 	11.6803990000 	; daa [13*20+11] = 	0.0062632873 	; daa [13*20+12] = 	1.7547518000 	;
	daa [14*20+ 0] = 	1.3722135000 	; daa [14*20+ 1] = 	0.1947621100 	; daa [14*20+ 2] = 	0.0000000000 	; daa [14*20+ 3] = 	0.0376957960 	; daa [14*20+ 4] = 	0.5790519200 	; daa [14*20+ 5] = 	2.5394101000 	; daa [14*20+ 6] = 	0.0000000000 	; daa [14*20+ 7] = 	0.1009966200 	; daa [14*20+ 8] = 	1.2853550000 	; daa [14*20+ 9] = 	0.0176227450 	; daa [14*20+10] = 	0.8794063100 	; daa [14*20+11] = 	0.2389308400 	; daa [14*20+12] = 	0.1126444900 	; daa [14*20+13] = 	0.0000000000 	;
	daa [15*20+ 0] = 	16.4255620000 	; daa [15*20+ 1] = 	0.4520599300 	; daa [15*20+ 2] = 	17.3949400000 	; daa [15*20+ 3] = 	1.1772153000 	; daa [15*20+ 4] = 	15.6784820000 	; daa [15*20+ 5] = 	0.7795206500 	; daa [15*20+ 6] = 	0.3415241200 	; daa [15*20+ 7] = 	4.3359462000 	; daa [15*20+ 8] = 	2.0429373000 	; daa [15*20+ 9] = 	0.1901514600 	; daa [15*20+10] = 	2.1567402000 	; daa [15*20+11] = 	0.6196357200 	; daa [15*20+12] = 	0.7021901600 	; daa [15*20+13] = 	2.1481878000 	; daa [15*20+14] = 	4.8778349000 	;
	daa [16*20+ 0] = 	21.8298350000 	; daa [16*20+ 1] = 	0.0618529460 	; daa [16*20+ 2] = 	3.6544258000 	; daa [16*20+ 3] = 	0.3573368300 	; daa [16*20+ 4] = 	1.0734293000 	; daa [16*20+ 5] = 	0.1978591300 	; daa [16*20+ 6] = 	0.3508714900 	; daa [16*20+ 7] = 	0.0000000000 	; daa [16*20+ 8] = 	0.5802435500 	; daa [16*20+ 9] = 	8.7261870000 	; daa [16*20+10] = 	1.1016160000 	; daa [16*20+11] = 	1.5713293000 	; daa [16*20+12] = 	12.7674590000 	; daa [16*20+13] = 	0.3759570700 	; daa [16*20+14] = 	1.6177783000 	; daa [16*20+15] = 	18.5483290000 	;
	daa [17*20+ 0] = 	0.0388509950 	; daa [17*20+ 1] = 	0.5680312900 	; daa [17*20+ 2] = 	0.0746537170 	; daa [17*20+ 3] = 	0.0000000000 	; daa [17*20+ 4] = 	2.8069938000 	; daa [17*20+ 5] = 	0.1870544300 	; daa [17*20+ 6] = 	0.0000000000 	; daa [17*20+ 7] = 	0.4081806100 	; daa [17*20+ 8] = 	0.1199210800 	; daa [17*20+ 9] = 	0.0000000000 	; daa [17*20+10] = 	0.8826221000 	; daa [17*20+11] = 	0.0879529950 	; daa [17*20+12] = 	0.7800139900 	; daa [17*20+13] = 	0.1247692200 	; daa [17*20+14] = 	0.0787807570 	; daa [17*20+15] = 	0.2628079600 	; daa [17*20+16] = 	0.0129885860 	;
	daa [18*20+ 0] = 	0.4987390700 	; daa [18*20+ 1] = 	0.0274117610 	; daa [18*20+ 2] = 	2.1714503000 	; daa [18*20+ 3] = 	0.0000000000 	; daa [18*20+ 4] = 	17.0609380000 	; daa [18*20+ 5] = 	0.5137322200 	; daa [18*20+ 6] = 	0.1415580800 	; daa [18*20+ 7] = 	0.1021521800 	; daa [18*20+ 8] = 	19.8978610000 	; daa [18*20+ 9] = 	0.3209350700 	; daa [18*20+10] = 	0.5302329100 	; daa [18*20+11] = 	0.1849909100 	; daa [18*20+12] = 	0.2771994500 	; daa [18*20+13] = 	26.1498020000 	; daa [18*20+14] = 	0.2148429100 	; daa [18*20+15] = 	2.1784705000 	; daa [18*20+16] = 	0.2439151200 	; daa [18*20+17] = 	0.9381859700 	;
	daa [19*20+ 0] = 	12.4667420000 	; daa [19*20+ 1] = 	0.1490978400 	; daa [19*20+ 2] = 	0.0000000000 	; daa [19*20+ 3] = 	0.0150521060 	; daa [19*20+ 4] = 	0.8049556100 	; daa [19*20+ 5] = 	0.1078445600 	; daa [19*20+ 6] = 	0.8138688600 	; daa [19*20+ 7] = 	0.4966372600 	; daa [19*20+ 8] = 	0.2221195100 	; daa [19*20+ 9] = 	89.0780560000 	; daa [19*20+10] = 	5.4663581000 	; daa [19*20+11] = 	0.2279529500 	; daa [19*20+12] = 	21.9966350000 	; daa [19*20+13] = 	1.0972105000 	; daa [19*20+14] = 	0.3323856600 	; daa [19*20+15] = 	0.3948493500 	; daa [19*20+16] = 	8.7280795000 	; daa [19*20+17] = 	0.0000000000 	; daa [19*20+18] = 	0.1573307000 	;
	
	f[ 0] = 0.0735129700 	;
	f[ 1] =	0.0380989040 	;
	f[ 2] =	0.0495533360 	;
	f[ 3] =	0.0377105010 	;
	f[ 4] =	0.0060036066 	;
	f[ 5] =	0.0446393400 	;
	f[ 6] =	0.0482043280 	;
	f[ 7] =	0.0594541550 	;
	f[ 8] =	0.0371070880 	;
	f[ 9] =	0.0493521990 	;
	f[10] =	0.1053065600 	;
	f[11] =	0.0329872380 	;
	f[12] =	0.0380746290 	;
	f[13] =	0.0452046050 	;
	f[14] =	0.0800846160 	;
	f[15] =	0.0669760020 	;
	f[16] =	0.0824670550 	;
	f[17] =	0.0306360110 	;
	f[18] =	0.0305666530 	;
	f[19] =	0.0440602030 	;

      }
      break;



    case DCMUT/* Tr178 */:
      {	
	daa [ 1*20+ 0] =  	0.0000000000 	;
	daa [ 2*20+ 0] =  	0.0683933870 	; daa [ 2*20+ 1] =  	8.0364021000 	;
	daa [ 3*20+ 0] =  	0.1117773500 	; daa [ 3*20+ 1] =  	0.0000000000 	; daa [ 3*20+ 2] =  	20.2089820000 	;
	daa [ 4*20+ 0] = 	3.5453650000 	; daa [ 4*20+ 1] = 	0.0000000000 	; daa [ 4*20+ 2] = 	0.0000000000 	; daa [ 4*20+ 3] = 	0.0000000000 	;
	daa [ 5*20+ 0] = 	0.0000000000 	; daa [ 5*20+ 1] = 	22.5879370000 	; daa [ 5*20+ 2] = 	2.7465517000 	; daa [ 5*20+ 3] = 	0.0000000000 	; daa [ 5*20+ 4] = 	0.0000000000 	;
	daa [ 6*20+ 0] = 	0.1345448000 	; daa [ 6*20+ 1] = 	0.0000000000 	; daa [ 6*20+ 2] = 	0.1528532900 	; daa [ 6*20+ 3] = 	2.9036975000 	; daa [ 6*20+ 4] = 	0.0000000000 	; daa [ 6*20+ 5] = 	17.7279410000 	;
	daa [ 7*20+ 0] = 	4.1606971000 	; daa [ 7*20+ 1] = 	1.3523170000 	; daa [ 7*20+ 2] = 	2.1563367000 	; daa [ 7*20+ 3] = 	0.2818011200 	; daa [ 7*20+ 4] = 	3.2109866000 	; daa [ 7*20+ 5] = 	0.0458514530 	; daa [ 7*20+ 6] = 	2.0402185000 	;
	daa [ 8*20+ 0] = 	0.3034850600 	; daa [ 8*20+ 1] = 	6.5069931000 	; daa [ 8*20+ 2] = 	8.1924424000 	; daa [ 8*20+ 3] = 	1.7653816000 	; daa [ 8*20+ 4] = 	1.2068326000 	; daa [ 8*20+ 5] = 	22.4641630000 	; daa [ 8*20+ 6] = 	0.5236849100 	; daa [ 8*20+ 7] = 	0.0000000000 	;
	daa [ 9*20+ 0] = 	0.6115705800 	; daa [ 9*20+ 1] = 	0.0813088850 	; daa [ 9*20+ 2] = 	0.3350396700 	; daa [ 9*20+ 3] = 	0.0000000000 	; daa [ 9*20+ 4] = 	0.4273957000 	; daa [ 9*20+ 5] = 	0.0000000000 	; daa [ 9*20+ 6] = 	0.0000000000 	; daa [ 9*20+ 7] = 	0.0000000000 	; daa [ 9*20+ 8] = 	0.0660608890 	;
	daa [10*20+ 0] = 	0.1993524600 	; daa [10*20+ 1] = 	0.3026023900 	; daa [10*20+ 2] = 	0.0877301280 	; daa [10*20+ 3] = 	0.0000000000 	; daa [10*20+ 4] = 	0.4976409800 	; daa [10*20+ 5] = 	0.4563656200 	; daa [10*20+ 6] = 	0.0003836538 	; daa [10*20+ 7] = 	0.0117606440 	; daa [10*20+ 8] = 	0.0567833280 	; daa [10*20+ 9] = 	4.2920109000 	;
	daa [11*20+ 0] = 	0.3124910100 	; daa [11*20+ 1] = 	3.4641029000 	; daa [11*20+ 2] = 	10.0145440000 	; daa [11*20+ 3] = 	5.6309688000 	; daa [11*20+ 4] = 	0.0000000000 	; daa [11*20+ 5] = 	20.2324890000 	; daa [11*20+ 6] = 	1.3099220000 	; daa [11*20+ 7] = 	0.1834055200 	; daa [11*20+ 8] = 	0.0000000000 	; daa [11*20+ 9] = 	0.0000000000 	; daa [11*20+10] = 	0.0535254310 	;
	daa [12*20+ 0] = 	3.2248261000 	; daa [12*20+ 1] = 	0.0000000000 	; daa [12*20+ 2] = 	0.0000000000 	; daa [12*20+ 3] = 	0.0000000000 	; daa [12*20+ 4] = 	0.2110897300 	; daa [12*20+ 5] = 	0.4174819000 	; daa [12*20+ 6] = 	0.0206172080 	; daa [12*20+ 7] = 	0.2843769200 	; daa [12*20+ 8] = 	0.0647551320 	; daa [12*20+ 9] = 	8.2557330000 	; daa [12*20+10] = 	10.1765210000 	; daa [12*20+11] = 	0.6765250900 	;
	daa [13*20+ 0] = 	0.1672473800 	; daa [13*20+ 1] = 	0.0000000000 	; daa [13*20+ 2] = 	0.0000000000 	; daa [13*20+ 3] = 	0.0000000000 	; daa [13*20+ 4] = 	4.0107301000 	; daa [13*20+ 5] = 	0.1055332800 	; daa [13*20+ 6] = 	0.0000000000 	; daa [13*20+ 7] = 	0.1900068900 	; daa [13*20+ 8] = 	0.1363253000 	; daa [13*20+ 9] = 	1.5164504000 	; daa [13*20+10] = 	4.5727298000 	; daa [13*20+11] = 	0.0000000000 	; daa [13*20+12] = 	0.4559076600 	;
	daa [14*20+ 0] = 	1.0775431000 	; daa [14*20+ 1] = 	0.2565294700 	; daa [14*20+ 2] = 	0.2859529200 	; daa [14*20+ 3] = 	0.3851802800 	; daa [14*20+ 4] = 	0.4675399500 	; daa [14*20+ 5] = 	2.2894906000 	; daa [14*20+ 6] = 	0.1280897100 	; daa [14*20+ 7] = 	0.1329750400 	; daa [14*20+ 8] = 	0.5979643000 	; daa [14*20+ 9] = 	0.0000000000 	; daa [14*20+10] = 	1.0357320000 	; daa [14*20+11] = 	2.9101161000 	; daa [14*20+12] = 	0.0000000000 	; daa [14*20+13] = 	0.2121771400 	;
	daa [15*20+ 0] = 	9.1473287000 	; daa [15*20+ 1] = 	0.2175744500 	; daa [15*20+ 2] = 	11.7853400000 	; daa [15*20+ 3] = 	2.5866470000 	; daa [15*20+ 4] = 	19.9951800000 	; daa [15*20+ 5] = 	0.5886149800 	; daa [15*20+ 6] = 	1.3846019000 	; daa [15*20+ 7] = 	3.4952482000 	; daa [15*20+ 8] = 	0.7189059400 	; daa [15*20+ 9] = 	0.0384500110 	; daa [15*20+10] = 	0.6476615200 	; daa [15*20+11] = 	0.9388045300 	; daa [15*20+12] = 	0.3250500500 	; daa [15*20+13] = 	2.4802397000 	; daa [15*20+14] = 	4.3358319000 	;
	daa [16*20+ 0] = 	14.3744260000 	; daa [16*20+ 1] = 	0.2841573500 	; daa [16*20+ 2] = 	3.6529068000 	; daa [16*20+ 3] = 	0.0000000000 	; daa [16*20+ 4] = 	0.2380579300 	; daa [16*20+ 5] = 	0.0000000000 	; daa [16*20+ 6] = 	0.1712433300 	; daa [16*20+ 7] = 	0.0894501140 	; daa [16*20+ 8] = 	0.1352112200 	; daa [16*20+ 9] = 	5.6404696000 	; daa [16*20+10] = 	0.5649653700 	; daa [16*20+11] = 	2.1723799000 	; daa [16*20+12] = 	8.9817934000 	; daa [16*20+13] = 	0.1859902100 	; daa [16*20+14] = 	1.5886027000 	; daa [16*20+15] = 	10.9044440000 	;
	daa [17*20+ 0] = 	0.0275751750 	; daa [17*20+ 1] = 	1.2549310000 	; daa [17*20+ 2] = 	0.0000000000 	; daa [17*20+ 3] = 	0.3913490500 	; daa [17*20+ 4] = 	4.1494038000 	; daa [17*20+ 5] = 	0.6024837600 	; daa [17*20+ 6] = 	0.0913683820 	; daa [17*20+ 7] = 	0.4827807200 	; daa [17*20+ 8] = 	0.0000000000 	; daa [17*20+ 9] = 	0.0000000000 	; daa [17*20+10] = 	0.2713619600 	; daa [17*20+11] = 	0.0000000000 	; daa [17*20+12] = 	0.2380579300 	; daa [17*20+13] = 	0.0638868700 	; daa [17*20+14] = 	0.0000000000 	; daa [17*20+15] = 	0.2122504100 	; daa [17*20+16] = 	0.0000000000 	;
	daa [18*20+ 0] = 	0.0384500110 	; daa [18*20+ 1] = 	1.4713300000 	; daa [18*20+ 2] = 	2.0571094000 	; daa [18*20+ 3] = 	1.3307232000 	; daa [18*20+ 4] = 	10.2753870000 	; daa [18*20+ 5] = 	0.3484486200 	; daa [18*20+ 6] = 	0.0000000000 	; daa [18*20+ 7] = 	0.0474973640 	; daa [18*20+ 8] = 	15.2327020000 	; daa [18*20+ 9] = 	0.0000000000 	; daa [18*20+10] = 	0.0832271540 	; daa [18*20+11] = 	0.0000000000 	; daa [18*20+12] = 	0.0000000000 	; daa [18*20+13] = 	9.8552647000 	; daa [18*20+14] = 	0.1739323700 	; daa [18*20+15] = 	0.9872608600 	; daa [18*20+16] = 	0.1970315400 	; daa [18*20+17] = 	0.2960927600 	;
	daa [19*20+ 0] = 	6.9866686000 	; daa [19*20+ 1] = 	0.0000000000 	; daa [19*20+ 2] = 	0.1821130000 	; daa [19*20+ 3] = 	0.4519090900 	; daa [19*20+ 4] = 	0.8601003300 	; daa [19*20+ 5] = 	0.2685685200 	; daa [19*20+ 6] = 	0.4593077300 	; daa [19*20+ 7] = 	1.0140986000 	; daa [19*20+ 8] = 	0.2273866100 	; daa [19*20+ 9] = 	40.5480190000 	; daa [19*20+10] = 	2.6696098000 	; daa [19*20+11] = 	0.0000000000 	; daa [19*20+12] = 	11.9397500000 	; daa [19*20+13] = 	0.9930697900 	; daa [19*20+14] = 	0.0896483970 	; daa [19*20+15] = 	0.0420804940 	; daa [19*20+16] = 	4.3832377000 	; daa [19*20+17] = 	0.0275751750 	; daa [19*20+18] = 	0.1261714400 	;
	
	
	f[ 0] = 0.1041960100 	;
	f[ 1] =	0.0066888879 	;
	f[ 2] =	0.0163019310 	;
	f[ 3] =	0.0067656652 	;
	f[ 4] =	0.0079279141 	;
	f[ 5] =	0.0113087610 	;
	f[ 6] =	0.0095931864 	;
	f[ 7] =	0.0696409470 	;
	f[ 8] =	0.0204028950 	;
	f[ 9] =	0.0896453950 	;
	f[10] =	0.2193883800 	;
	f[11] =	0.0096752587 	;
	f[12] =	0.0479209250 	;
	f[13] =	0.0729847290 	;
	f[14] =	0.0373256630 	;
	f[15] =	0.0611186710 	;
	f[16] =	0.0767891750 	;
	f[17] =	0.0306169190 	;
	f[18] =	0.0296479370 	;
	f[19] =	0.0720607550 	;
      }
      break;



    case JTT:
      {
	daa[ 1*20+ 0] =   58.00; daa[ 2*20+ 0] =   54.00; daa[ 2*20+ 1] =   45.00; daa[ 3*20+ 0] =   81.00;
	daa[ 3*20+ 1] =   16.00; daa[ 3*20+ 2] =  528.00; daa[ 4*20+ 0] =   56.00; daa[ 4*20+ 1] =  113.00;
	daa[ 4*20+ 2] =   34.00; daa[ 4*20+ 3] =   10.00; daa[ 5*20+ 0] =   57.00; daa[ 5*20+ 1] =  310.00;
	daa[ 5*20+ 2] =   86.00; daa[ 5*20+ 3] =   49.00; daa[ 5*20+ 4] =    9.00; daa[ 6*20+ 0] =  105.00;
	daa[ 6*20+ 1] =   29.00; daa[ 6*20+ 2] =   58.00; daa[ 6*20+ 3] =  767.00; daa[ 6*20+ 4] =    5.00;
	daa[ 6*20+ 5] =  323.00; daa[ 7*20+ 0] =  179.00; daa[ 7*20+ 1] =  137.00; daa[ 7*20+ 2] =   81.00;
	daa[ 7*20+ 3] =  130.00; daa[ 7*20+ 4] =   59.00; daa[ 7*20+ 5] =   26.00; daa[ 7*20+ 6] =  119.00;
	daa[ 8*20+ 0] =   27.00; daa[ 8*20+ 1] =  328.00; daa[ 8*20+ 2] =  391.00; daa[ 8*20+ 3] =  112.00;
	daa[ 8*20+ 4] =   69.00; daa[ 8*20+ 5] =  597.00; daa[ 8*20+ 6] =   26.00; daa[ 8*20+ 7] =   23.00;
	daa[ 9*20+ 0] =   36.00; daa[ 9*20+ 1] =   22.00; daa[ 9*20+ 2] =   47.00; daa[ 9*20+ 3] =   11.00;
	daa[ 9*20+ 4] =   17.00; daa[ 9*20+ 5] =    9.00; daa[ 9*20+ 6] =   12.00; daa[ 9*20+ 7] =    6.00;
	daa[ 9*20+ 8] =   16.00; daa[10*20+ 0] =   30.00; daa[10*20+ 1] =   38.00; daa[10*20+ 2] =   12.00;
	daa[10*20+ 3] =    7.00; daa[10*20+ 4] =   23.00; daa[10*20+ 5] =   72.00; daa[10*20+ 6] =    9.00;
	daa[10*20+ 7] =    6.00; daa[10*20+ 8] =   56.00; daa[10*20+ 9] =  229.00; daa[11*20+ 0] =   35.00;
	daa[11*20+ 1] =  646.00; daa[11*20+ 2] =  263.00; daa[11*20+ 3] =   26.00; daa[11*20+ 4] =    7.00;
	daa[11*20+ 5] =  292.00; daa[11*20+ 6] =  181.00; daa[11*20+ 7] =   27.00; daa[11*20+ 8] =   45.00;
	daa[11*20+ 9] =   21.00; daa[11*20+10] =   14.00; daa[12*20+ 0] =   54.00; daa[12*20+ 1] =   44.00;
	daa[12*20+ 2] =   30.00; daa[12*20+ 3] =   15.00; daa[12*20+ 4] =   31.00; daa[12*20+ 5] =   43.00;
	daa[12*20+ 6] =   18.00; daa[12*20+ 7] =   14.00; daa[12*20+ 8] =   33.00; daa[12*20+ 9] =  479.00;
	daa[12*20+10] =  388.00; daa[12*20+11] =   65.00; daa[13*20+ 0] =   15.00; daa[13*20+ 1] =    5.00;
	daa[13*20+ 2] =   10.00; daa[13*20+ 3] =    4.00; daa[13*20+ 4] =   78.00; daa[13*20+ 5] =    4.00;
	daa[13*20+ 6] =    5.00; daa[13*20+ 7] =    5.00; daa[13*20+ 8] =   40.00; daa[13*20+ 9] =   89.00;
	daa[13*20+10] =  248.00; daa[13*20+11] =    4.00; daa[13*20+12] =   43.00; daa[14*20+ 0] =  194.00;
	daa[14*20+ 1] =   74.00; daa[14*20+ 2] =   15.00; daa[14*20+ 3] =   15.00; daa[14*20+ 4] =   14.00;
	daa[14*20+ 5] =  164.00; daa[14*20+ 6] =   18.00; daa[14*20+ 7] =   24.00; daa[14*20+ 8] =  115.00;
	daa[14*20+ 9] =   10.00; daa[14*20+10] =  102.00; daa[14*20+11] =   21.00; daa[14*20+12] =   16.00;
	daa[14*20+13] =   17.00; daa[15*20+ 0] =  378.00; daa[15*20+ 1] =  101.00; daa[15*20+ 2] =  503.00;
	daa[15*20+ 3] =   59.00; daa[15*20+ 4] =  223.00; daa[15*20+ 5] =   53.00; daa[15*20+ 6] =   30.00;
	daa[15*20+ 7] =  201.00; daa[15*20+ 8] =   73.00; daa[15*20+ 9] =   40.00; daa[15*20+10] =   59.00;
	daa[15*20+11] =   47.00; daa[15*20+12] =   29.00; daa[15*20+13] =   92.00; daa[15*20+14] =  285.00;
	daa[16*20+ 0] =  475.00; daa[16*20+ 1] =   64.00; daa[16*20+ 2] =  232.00; daa[16*20+ 3] =   38.00;
	daa[16*20+ 4] =   42.00; daa[16*20+ 5] =   51.00; daa[16*20+ 6] =   32.00; daa[16*20+ 7] =   33.00;
	daa[16*20+ 8] =   46.00; daa[16*20+ 9] =  245.00; daa[16*20+10] =   25.00; daa[16*20+11] =  103.00;
	daa[16*20+12] =  226.00; daa[16*20+13] =   12.00; daa[16*20+14] =  118.00; daa[16*20+15] =  477.00;
	daa[17*20+ 0] =    9.00; daa[17*20+ 1] =  126.00; daa[17*20+ 2] =    8.00; daa[17*20+ 3] =    4.00;
	daa[17*20+ 4] =  115.00; daa[17*20+ 5] =   18.00; daa[17*20+ 6] =   10.00; daa[17*20+ 7] =   55.00;
	daa[17*20+ 8] =    8.00; daa[17*20+ 9] =    9.00; daa[17*20+10] =   52.00; daa[17*20+11] =   10.00;
	daa[17*20+12] =   24.00; daa[17*20+13] =   53.00; daa[17*20+14] =    6.00; daa[17*20+15] =   35.00;
	daa[17*20+16] =   12.00; daa[18*20+ 0] =   11.00; daa[18*20+ 1] =   20.00; daa[18*20+ 2] =   70.00;
	daa[18*20+ 3] =   46.00; daa[18*20+ 4] =  209.00; daa[18*20+ 5] =   24.00; daa[18*20+ 6] =    7.00;
	daa[18*20+ 7] =    8.00; daa[18*20+ 8] =  573.00; daa[18*20+ 9] =   32.00; daa[18*20+10] =   24.00;
	daa[18*20+11] =    8.00; daa[18*20+12] =   18.00; daa[18*20+13] =  536.00; daa[18*20+14] =   10.00;
	daa[18*20+15] =   63.00; daa[18*20+16] =   21.00; daa[18*20+17] =   71.00; daa[19*20+ 0] =  298.00;
	daa[19*20+ 1] =   17.00; daa[19*20+ 2] =   16.00; daa[19*20+ 3] =   31.00; daa[19*20+ 4] =   62.00;
	daa[19*20+ 5] =   20.00; daa[19*20+ 6] =   45.00; daa[19*20+ 7] =   47.00; daa[19*20+ 8] =   11.00;
	daa[19*20+ 9] =  961.00; daa[19*20+10] =  180.00; daa[19*20+11] =   14.00; daa[19*20+12] =  323.00;
	daa[19*20+13] =   62.00; daa[19*20+14] =   23.00; daa[19*20+15] =   38.00; daa[19*20+16] =  112.00;
	daa[19*20+17] =   25.00; daa[19*20+18] =   16.00;

	f[ 0] = 0.076748; f[ 1] = 0.051691; f[ 2] = 0.042645; f[ 3] = 0.051544;
	f[ 4] = 0.019803; f[ 5] = 0.040752; f[ 6] = 0.061830; f[ 7] = 0.073152;
	f[ 8] = 0.022944; f[ 9] = 0.053761; f[10] = 0.091904; f[11] = 0.058676;
	f[12] = 0.023826; f[13] = 0.040126; f[14] = 0.050901; f[15] = 0.068765;
	f[16] = 0.058565; f[17] = 0.014261; f[18] = 0.032102; f[19] = 0.066005;
      }
      break;
    case  MTREV:
      {
	daa[ 1*20+ 0] =   23.18; daa[ 2*20+ 0] =   26.95; daa[ 2*20+ 1] =   13.24; daa[ 3*20+ 0] =   17.67;
	daa[ 3*20+ 1] =    1.90; daa[ 3*20+ 2] =  794.38; daa[ 4*20+ 0] =   59.93; daa[ 4*20+ 1] =  103.33;
	daa[ 4*20+ 2] =   58.94; daa[ 4*20+ 3] =    1.90; daa[ 5*20+ 0] =    1.90; daa[ 5*20+ 1] =  220.99;
	daa[ 5*20+ 2] =  173.56; daa[ 5*20+ 3] =   55.28; daa[ 5*20+ 4] =   75.24; daa[ 6*20+ 0] =    9.77;
	daa[ 6*20+ 1] =    1.90; daa[ 6*20+ 2] =   63.05; daa[ 6*20+ 3] =  583.55; daa[ 6*20+ 4] =    1.90;
	daa[ 6*20+ 5] =  313.56; daa[ 7*20+ 0] =  120.71; daa[ 7*20+ 1] =   23.03; daa[ 7*20+ 2] =   53.30;
	daa[ 7*20+ 3] =   56.77; daa[ 7*20+ 4] =   30.71; daa[ 7*20+ 5] =    6.75; daa[ 7*20+ 6] =   28.28;
	daa[ 8*20+ 0] =   13.90; daa[ 8*20+ 1] =  165.23; daa[ 8*20+ 2] =  496.13; daa[ 8*20+ 3] =  113.99;
	daa[ 8*20+ 4] =  141.49; daa[ 8*20+ 5] =  582.40; daa[ 8*20+ 6] =   49.12; daa[ 8*20+ 7] =    1.90;
	daa[ 9*20+ 0] =   96.49; daa[ 9*20+ 1] =    1.90; daa[ 9*20+ 2] =   27.10; daa[ 9*20+ 3] =    4.34;
	daa[ 9*20+ 4] =   62.73; daa[ 9*20+ 5] =    8.34; daa[ 9*20+ 6] =    3.31; daa[ 9*20+ 7] =    5.98;
	daa[ 9*20+ 8] =   12.26; daa[10*20+ 0] =   25.46; daa[10*20+ 1] =   15.58; daa[10*20+ 2] =   15.16;
	daa[10*20+ 3] =    1.90; daa[10*20+ 4] =   25.65; daa[10*20+ 5] =   39.70; daa[10*20+ 6] =    1.90;
	daa[10*20+ 7] =    2.41; daa[10*20+ 8] =   11.49; daa[10*20+ 9] =  329.09; daa[11*20+ 0] =    8.36;
	daa[11*20+ 1] =  141.40; daa[11*20+ 2] =  608.70; daa[11*20+ 3] =    2.31; daa[11*20+ 4] =    1.90;
	daa[11*20+ 5] =  465.58; daa[11*20+ 6] =  313.86; daa[11*20+ 7] =   22.73; daa[11*20+ 8] =  127.67;
	daa[11*20+ 9] =   19.57; daa[11*20+10] =   14.88; daa[12*20+ 0] =  141.88; daa[12*20+ 1] =    1.90;
	daa[12*20+ 2] =   65.41; daa[12*20+ 3] =    1.90; daa[12*20+ 4] =    6.18; daa[12*20+ 5] =   47.37;
	daa[12*20+ 6] =    1.90; daa[12*20+ 7] =    1.90; daa[12*20+ 8] =   11.97; daa[12*20+ 9] =  517.98;
	daa[12*20+10] =  537.53; daa[12*20+11] =   91.37; daa[13*20+ 0] =    6.37; daa[13*20+ 1] =    4.69;
	daa[13*20+ 2] =   15.20; daa[13*20+ 3] =    4.98; daa[13*20+ 4] =   70.80; daa[13*20+ 5] =   19.11;
	daa[13*20+ 6] =    2.67; daa[13*20+ 7] =    1.90; daa[13*20+ 8] =   48.16; daa[13*20+ 9] =   84.67;
	daa[13*20+10] =  216.06; daa[13*20+11] =    6.44; daa[13*20+12] =   90.82; daa[14*20+ 0] =   54.31;
	daa[14*20+ 1] =   23.64; daa[14*20+ 2] =   73.31; daa[14*20+ 3] =   13.43; daa[14*20+ 4] =   31.26;
	daa[14*20+ 5] =  137.29; daa[14*20+ 6] =   12.83; daa[14*20+ 7] =    1.90; daa[14*20+ 8] =   60.97;
	daa[14*20+ 9] =   20.63; daa[14*20+10] =   40.10; daa[14*20+11] =   50.10; daa[14*20+12] =   18.84;
	daa[14*20+13] =   17.31; daa[15*20+ 0] =  387.86; daa[15*20+ 1] =    6.04; daa[15*20+ 2] =  494.39;
	daa[15*20+ 3] =   69.02; daa[15*20+ 4] =  277.05; daa[15*20+ 5] =   54.11; daa[15*20+ 6] =   54.71;
	daa[15*20+ 7] =  125.93; daa[15*20+ 8] =   77.46; daa[15*20+ 9] =   47.70; daa[15*20+10] =   73.61;
	daa[15*20+11] =  105.79; daa[15*20+12] =  111.16; daa[15*20+13] =   64.29; daa[15*20+14] =  169.90;
	daa[16*20+ 0] =  480.72; daa[16*20+ 1] =    2.08; daa[16*20+ 2] =  238.46; daa[16*20+ 3] =   28.01;
	daa[16*20+ 4] =  179.97; daa[16*20+ 5] =   94.93; daa[16*20+ 6] =   14.82; daa[16*20+ 7] =   11.17;
	daa[16*20+ 8] =   44.78; daa[16*20+ 9] =  368.43; daa[16*20+10] =  126.40; daa[16*20+11] =  136.33;
	daa[16*20+12] =  528.17; daa[16*20+13] =   33.85; daa[16*20+14] =  128.22; daa[16*20+15] =  597.21;
	daa[17*20+ 0] =    1.90; daa[17*20+ 1] =   21.95; daa[17*20+ 2] =   10.68; daa[17*20+ 3] =   19.86;
	daa[17*20+ 4] =   33.60; daa[17*20+ 5] =    1.90; daa[17*20+ 6] =    1.90; daa[17*20+ 7] =   10.92;
	daa[17*20+ 8] =    7.08; daa[17*20+ 9] =    1.90; daa[17*20+10] =   32.44; daa[17*20+11] =   24.00;
	daa[17*20+12] =   21.71; daa[17*20+13] =    7.84; daa[17*20+14] =    4.21; daa[17*20+15] =   38.58;
	daa[17*20+16] =    9.99; daa[18*20+ 0] =    6.48; daa[18*20+ 1] =    1.90; daa[18*20+ 2] =  191.36;
	daa[18*20+ 3] =   21.21; daa[18*20+ 4] =  254.77; daa[18*20+ 5] =   38.82; daa[18*20+ 6] =   13.12;
	daa[18*20+ 7] =    3.21; daa[18*20+ 8] =  670.14; daa[18*20+ 9] =   25.01; daa[18*20+10] =   44.15;
	daa[18*20+11] =   51.17; daa[18*20+12] =   39.96; daa[18*20+13] =  465.58; daa[18*20+14] =   16.21;
	daa[18*20+15] =   64.92; daa[18*20+16] =   38.73; daa[18*20+17] =   26.25; daa[19*20+ 0] =  195.06;
	daa[19*20+ 1] =    7.64; daa[19*20+ 2] =    1.90; daa[19*20+ 3] =    1.90; daa[19*20+ 4] =    1.90;
	daa[19*20+ 5] =   19.00; daa[19*20+ 6] =   21.14; daa[19*20+ 7] =    2.53; daa[19*20+ 8] =    1.90;
	daa[19*20+ 9] = 1222.94; daa[19*20+10] =   91.67; daa[19*20+11] =    1.90; daa[19*20+12] =  387.54;
	daa[19*20+13] =    6.35; daa[19*20+14] =    8.23; daa[19*20+15] =    1.90; daa[19*20+16] =  204.54;
	daa[19*20+17] =    5.37; daa[19*20+18] =    1.90;
	
  
	f[ 0] = 0.072000; f[ 1] = 0.019000; f[ 2] = 0.039000; f[ 3] = 0.019000;
	f[ 4] = 0.006000; f[ 5] = 0.025000; f[ 6] = 0.024000; f[ 7] = 0.056000;
	f[ 8] = 0.028000; f[ 9] = 0.088000; f[10] = 0.169000; f[11] = 0.023000;
	f[12] = 0.054000; f[13] = 0.061000; f[14] = 0.054000; f[15] = 0.072000;
	f[16] = 0.086000; f[17] = 0.029000; f[18] = 0.033000; f[19] = 0.043000;
      }
      break;
    case WAG:
      {
	daa[ 1*20+ 0] =  55.15710; daa[ 2*20+ 0] =  50.98480; daa[ 2*20+ 1] =  63.53460; 
	daa[ 3*20+ 0] =  73.89980; daa[ 3*20+ 1] =  14.73040; daa[ 3*20+ 2] = 542.94200; 
	daa[ 4*20+ 0] = 102.70400; daa[ 4*20+ 1] =  52.81910; daa[ 4*20+ 2] =  26.52560; 
	daa[ 4*20+ 3] =   3.02949; daa[ 5*20+ 0] =  90.85980; daa[ 5*20+ 1] = 303.55000; 
	daa[ 5*20+ 2] = 154.36400; daa[ 5*20+ 3] =  61.67830; daa[ 5*20+ 4] =   9.88179; 
	daa[ 6*20+ 0] = 158.28500; daa[ 6*20+ 1] =  43.91570; daa[ 6*20+ 2] =  94.71980; 
	daa[ 6*20+ 3] = 617.41600; daa[ 6*20+ 4] =   2.13520; daa[ 6*20+ 5] = 546.94700; 
	daa[ 7*20+ 0] = 141.67200; daa[ 7*20+ 1] =  58.46650; daa[ 7*20+ 2] = 112.55600; 
	daa[ 7*20+ 3] =  86.55840; daa[ 7*20+ 4] =  30.66740; daa[ 7*20+ 5] =  33.00520; 
	daa[ 7*20+ 6] =  56.77170; daa[ 8*20+ 0] =  31.69540; daa[ 8*20+ 1] = 213.71500; 
	daa[ 8*20+ 2] = 395.62900; daa[ 8*20+ 3] =  93.06760; daa[ 8*20+ 4] =  24.89720; 
	daa[ 8*20+ 5] = 429.41100; daa[ 8*20+ 6] =  57.00250; daa[ 8*20+ 7] =  24.94100; 
	daa[ 9*20+ 0] =  19.33350; daa[ 9*20+ 1] =  18.69790; daa[ 9*20+ 2] =  55.42360; 
	daa[ 9*20+ 3] =   3.94370; daa[ 9*20+ 4] =  17.01350; daa[ 9*20+ 5] =  11.39170; 
	daa[ 9*20+ 6] =  12.73950; daa[ 9*20+ 7] =   3.04501; daa[ 9*20+ 8] =  13.81900; 
	daa[10*20+ 0] =  39.79150; daa[10*20+ 1] =  49.76710; daa[10*20+ 2] =  13.15280; 
	daa[10*20+ 3] =   8.48047; daa[10*20+ 4] =  38.42870; daa[10*20+ 5] =  86.94890; 
	daa[10*20+ 6] =  15.42630; daa[10*20+ 7] =   6.13037; daa[10*20+ 8] =  49.94620; 
	daa[10*20+ 9] = 317.09700; daa[11*20+ 0] =  90.62650; daa[11*20+ 1] = 535.14200; 
	daa[11*20+ 2] = 301.20100; daa[11*20+ 3] =  47.98550; daa[11*20+ 4] =   7.40339; 
	daa[11*20+ 5] = 389.49000; daa[11*20+ 6] = 258.44300; daa[11*20+ 7] =  37.35580; 
	daa[11*20+ 8] =  89.04320; daa[11*20+ 9] =  32.38320; daa[11*20+10] =  25.75550; 
	daa[12*20+ 0] =  89.34960; daa[12*20+ 1] =  68.31620; daa[12*20+ 2] =  19.82210; 
	daa[12*20+ 3] =  10.37540; daa[12*20+ 4] =  39.04820; daa[12*20+ 5] = 154.52600; 
	daa[12*20+ 6] =  31.51240; daa[12*20+ 7] =  17.41000; daa[12*20+ 8] =  40.41410; 
	daa[12*20+ 9] = 425.74600; daa[12*20+10] = 485.40200; daa[12*20+11] =  93.42760; 
	daa[13*20+ 0] =  21.04940; daa[13*20+ 1] =  10.27110; daa[13*20+ 2] =   9.61621; 
	daa[13*20+ 3] =   4.67304; daa[13*20+ 4] =  39.80200; daa[13*20+ 5] =   9.99208; 
	daa[13*20+ 6] =   8.11339; daa[13*20+ 7] =   4.99310; daa[13*20+ 8] =  67.93710; 
	daa[13*20+ 9] = 105.94700; daa[13*20+10] = 211.51700; daa[13*20+11] =   8.88360; 
	daa[13*20+12] = 119.06300; daa[14*20+ 0] = 143.85500; daa[14*20+ 1] =  67.94890; 
	daa[14*20+ 2] =  19.50810; daa[14*20+ 3] =  42.39840; daa[14*20+ 4] =  10.94040; 
	daa[14*20+ 5] =  93.33720; daa[14*20+ 6] =  68.23550; daa[14*20+ 7] =  24.35700; 
	daa[14*20+ 8] =  69.61980; daa[14*20+ 9] =   9.99288; daa[14*20+10] =  41.58440; 
	daa[14*20+11] =  55.68960; daa[14*20+12] =  17.13290; daa[14*20+13] =  16.14440; 
	daa[15*20+ 0] = 337.07900; daa[15*20+ 1] = 122.41900; daa[15*20+ 2] = 397.42300; 
	daa[15*20+ 3] = 107.17600; daa[15*20+ 4] = 140.76600; daa[15*20+ 5] = 102.88700; 
	daa[15*20+ 6] =  70.49390; daa[15*20+ 7] = 134.18200; daa[15*20+ 8] =  74.01690; 
	daa[15*20+ 9] =  31.94400; daa[15*20+10] =  34.47390; daa[15*20+11] =  96.71300; 
	daa[15*20+12] =  49.39050; daa[15*20+13] =  54.59310; daa[15*20+14] = 161.32800; 
	daa[16*20+ 0] = 212.11100; daa[16*20+ 1] =  55.44130; daa[16*20+ 2] = 203.00600; 
	daa[16*20+ 3] =  37.48660; daa[16*20+ 4] =  51.29840; daa[16*20+ 5] =  85.79280; 
	daa[16*20+ 6] =  82.27650; daa[16*20+ 7] =  22.58330; daa[16*20+ 8] =  47.33070; 
	daa[16*20+ 9] = 145.81600; daa[16*20+10] =  32.66220; daa[16*20+11] = 138.69800; 
	daa[16*20+12] = 151.61200; daa[16*20+13] =  17.19030; daa[16*20+14] =  79.53840; 
	daa[16*20+15] = 437.80200; daa[17*20+ 0] =  11.31330; daa[17*20+ 1] = 116.39200; 
	daa[17*20+ 2] =   7.19167; daa[17*20+ 3] =  12.97670; daa[17*20+ 4] =  71.70700; 
	daa[17*20+ 5] =  21.57370; daa[17*20+ 6] =  15.65570; daa[17*20+ 7] =  33.69830; 
	daa[17*20+ 8] =  26.25690; daa[17*20+ 9] =  21.24830; daa[17*20+10] =  66.53090; 
	daa[17*20+11] =  13.75050; daa[17*20+12] =  51.57060; daa[17*20+13] = 152.96400; 
	daa[17*20+14] =  13.94050; daa[17*20+15] =  52.37420; daa[17*20+16] =  11.08640; 
	daa[18*20+ 0] =  24.07350; daa[18*20+ 1] =  38.15330; daa[18*20+ 2] = 108.60000; 
	daa[18*20+ 3] =  32.57110; daa[18*20+ 4] =  54.38330; daa[18*20+ 5] =  22.77100; 
	daa[18*20+ 6] =  19.63030; daa[18*20+ 7] =  10.36040; daa[18*20+ 8] = 387.34400; 
	daa[18*20+ 9] =  42.01700; daa[18*20+10] =  39.86180; daa[18*20+11] =  13.32640; 
	daa[18*20+12] =  42.84370; daa[18*20+13] = 645.42800; daa[18*20+14] =  21.60460; 
	daa[18*20+15] =  78.69930; daa[18*20+16] =  29.11480; daa[18*20+17] = 248.53900; 
	daa[19*20+ 0] = 200.60100; daa[19*20+ 1] =  25.18490; daa[19*20+ 2] =  19.62460; 
	daa[19*20+ 3] =  15.23350; daa[19*20+ 4] = 100.21400; daa[19*20+ 5] =  30.12810; 
	daa[19*20+ 6] =  58.87310; daa[19*20+ 7] =  18.72470; daa[19*20+ 8] =  11.83580; 
	daa[19*20+ 9] = 782.13000; daa[19*20+10] = 180.03400; daa[19*20+11] =  30.54340; 
	daa[19*20+12] = 205.84500; daa[19*20+13] =  64.98920; daa[19*20+14] =  31.48870; 
	daa[19*20+15] =  23.27390; daa[19*20+16] = 138.82300; daa[19*20+17] =  36.53690; 
	daa[19*20+18] =  31.47300; 
  
	f[0] = 0.0866279; f[1] =  0.043972; f[2] =  0.0390894; f[3] =  0.0570451;
	f[4] =  0.0193078; f[5] =  0.0367281; f[6] =  0.0580589; f[7] =  0.0832518;
	f[8] =  0.0244313; f[9] =  0.048466; f[10] =  0.086209; f[11] = 0.0620286;
	f[12] = 0.0195027; f[13] =  0.0384319; f[14] =  0.0457631; f[15] = 0.0695179;
	f[16] =  0.0610127; f[17] =  0.0143859; f[18] =  0.0352742; f[19] =  0.0708956;
      }
      break;
    case RTREV:
      {
	daa[1*20+0]= 34;         daa[2*20+0]= 51;         daa[2*20+1]= 35;         daa[3*20+0]= 10;         
	daa[3*20+1]= 30;         daa[3*20+2]= 384;        daa[4*20+0]= 439;        daa[4*20+1]= 92;         
	daa[4*20+2]= 128;        daa[4*20+3]= 1;          daa[5*20+0]= 32;         daa[5*20+1]= 221;        
	daa[5*20+2]= 236;        daa[5*20+3]= 78;         daa[5*20+4]= 70;         daa[6*20+0]= 81;         
	daa[6*20+1]= 10;         daa[6*20+2]= 79;         daa[6*20+3]= 542;        daa[6*20+4]= 1;          
	daa[6*20+5]= 372;        daa[7*20+0]= 135;        daa[7*20+1]= 41;         daa[7*20+2]= 94;         
	daa[7*20+3]= 61;         daa[7*20+4]= 48;         daa[7*20+5]= 18;         daa[7*20+6]= 70;         
	daa[8*20+0]= 30;         daa[8*20+1]= 90;         daa[8*20+2]= 320;        daa[8*20+3]= 91;         
	daa[8*20+4]= 124;        daa[8*20+5]= 387;        daa[8*20+6]= 34;         daa[8*20+7]= 68;         
	daa[9*20+0]= 1;          daa[9*20+1]= 24;         daa[9*20+2]= 35;         daa[9*20+3]= 1;          
	daa[9*20+4]= 104;        daa[9*20+5]= 33;         daa[9*20+6]= 1;          daa[9*20+7]= 1;          
	daa[9*20+8]= 34;         daa[10*20+0]= 45;        daa[10*20+1]= 18;        daa[10*20+2]= 15;        
	daa[10*20+3]= 5;         daa[10*20+4]= 110;       daa[10*20+5]= 54;        daa[10*20+6]= 21;        
	daa[10*20+7]= 3;         daa[10*20+8]= 51;        daa[10*20+9]= 385;       daa[11*20+0]= 38;        
	daa[11*20+1]= 593;       daa[11*20+2]= 123;       daa[11*20+3]= 20;        daa[11*20+4]= 16;        
	daa[11*20+5]= 309;       daa[11*20+6]= 141;       daa[11*20+7]= 30;        daa[11*20+8]= 76;        
	daa[11*20+9]= 34;        daa[11*20+10]= 23;       daa[12*20+0]= 235;       daa[12*20+1]= 57;        
	daa[12*20+2]= 1;         daa[12*20+3]= 1;         daa[12*20+4]= 156;       daa[12*20+5]= 158;       
	daa[12*20+6]= 1;         daa[12*20+7]= 37;        daa[12*20+8]= 116;       daa[12*20+9]= 375;       
	daa[12*20+10]= 581;      daa[12*20+11]= 134;      daa[13*20+0]= 1;         daa[13*20+1]= 7;         
	daa[13*20+2]= 49;        daa[13*20+3]= 1;         daa[13*20+4]= 70;        daa[13*20+5]= 1;         
	daa[13*20+6]= 1;         daa[13*20+7]= 7;         daa[13*20+8]= 141;       daa[13*20+9]= 64;        
	daa[13*20+10]= 179;      daa[13*20+11]= 14;       daa[13*20+12]= 247;      daa[14*20+0]= 97;        
	daa[14*20+1]= 24;        daa[14*20+2]= 33;        daa[14*20+3]= 55;        daa[14*20+4]= 1;         
	daa[14*20+5]= 68;        daa[14*20+6]= 52;        daa[14*20+7]= 17;        daa[14*20+8]= 44;        
	daa[14*20+9]= 10;        daa[14*20+10]= 22;       daa[14*20+11]= 43;       daa[14*20+12]= 1;        
	daa[14*20+13]= 11;       daa[15*20+0]= 460;       daa[15*20+1]= 102;       daa[15*20+2]= 294;       
	daa[15*20+3]= 136;       daa[15*20+4]= 75;        daa[15*20+5]= 225;       daa[15*20+6]= 95;        
	daa[15*20+7]= 152;       daa[15*20+8]= 183;       daa[15*20+9]= 4;         daa[15*20+10]= 24;       
	daa[15*20+11]= 77;       daa[15*20+12]= 1;        daa[15*20+13]= 20;       daa[15*20+14]= 134;      
	daa[16*20+0]= 258;       daa[16*20+1]= 64;        daa[16*20+2]= 148;       daa[16*20+3]= 55;        
	daa[16*20+4]= 117;       daa[16*20+5]= 146;       daa[16*20+6]= 82;        daa[16*20+7]= 7;         
	daa[16*20+8]= 49;        daa[16*20+9]= 72;        daa[16*20+10]= 25;       daa[16*20+11]= 110;      
	daa[16*20+12]= 131;      daa[16*20+13]= 69;       daa[16*20+14]= 62;       daa[16*20+15]= 671;      
	daa[17*20+0]= 5;         daa[17*20+1]= 13;        daa[17*20+2]= 16;        daa[17*20+3]= 1;         
	daa[17*20+4]= 55;        daa[17*20+5]= 10;        daa[17*20+6]= 17;        daa[17*20+7]= 23;        
	daa[17*20+8]= 48;        daa[17*20+9]= 39;        daa[17*20+10]= 47;       daa[17*20+11]= 6;        
	daa[17*20+12]= 111;      daa[17*20+13]= 182;      daa[17*20+14]= 9;        daa[17*20+15]= 14;       
	daa[17*20+16]= 1;        daa[18*20+0]= 55;        daa[18*20+1]= 47;        daa[18*20+2]= 28;        
	daa[18*20+3]= 1;         daa[18*20+4]= 131;       daa[18*20+5]= 45;        daa[18*20+6]= 1;         
	daa[18*20+7]= 21;        daa[18*20+8]= 307;       daa[18*20+9]= 26;        daa[18*20+10]= 64;       
	daa[18*20+11]= 1;        daa[18*20+12]= 74;       daa[18*20+13]= 1017;     daa[18*20+14]= 14;       
	daa[18*20+15]= 31;       daa[18*20+16]= 34;       daa[18*20+17]= 176;      daa[19*20+0]= 197;       
	daa[19*20+1]= 29;        daa[19*20+2]= 21;        daa[19*20+3]= 6;         daa[19*20+4]= 295;       
	daa[19*20+5]= 36;        daa[19*20+6]= 35;        daa[19*20+7]= 3;         daa[19*20+8]= 1;         
	daa[19*20+9]= 1048;      daa[19*20+10]= 112;      daa[19*20+11]= 19;       daa[19*20+12]= 236;      
	daa[19*20+13]= 92;       daa[19*20+14]= 25;       daa[19*20+15]= 39;       daa[19*20+16]= 196;      
	daa[19*20+17]= 26;       daa[19*20+18]= 59;       
			
	f[0]= 0.0646;           f[1]= 0.0453;           f[2]= 0.0376;           f[3]= 0.0422;           
	f[4]= 0.0114;           f[5]= 0.0606;           f[6]= 0.0607;           f[7]= 0.0639;           
	f[8]= 0.0273;           f[9]= 0.0679;           f[10]= 0.1018;          f[11]= 0.0751;          
	f[12]= 0.015;           f[13]= 0.0287;          f[14]= 0.0681;          f[15]= 0.0488;          
	f[16]= 0.0622;          f[17]= 0.0251;          f[18]= 0.0318;          f[19]= 0.0619;
      }
      break;
    case CPREV:
      {
	daa[1*20+0]= 105;        daa[2*20+0]= 227;        daa[2*20+1]= 357;        daa[3*20+0]= 175;        
	daa[3*20+1]= 43;         daa[3*20+2]= 4435;       daa[4*20+0]= 669;        daa[4*20+1]= 823;        
	daa[4*20+2]= 538;        daa[4*20+3]= 10;         daa[5*20+0]= 157;        daa[5*20+1]= 1745;       
	daa[5*20+2]= 768;        daa[5*20+3]= 400;        daa[5*20+4]= 10;         daa[6*20+0]= 499;        
	daa[6*20+1]= 152;        daa[6*20+2]= 1055;       daa[6*20+3]= 3691;       daa[6*20+4]= 10;         
	daa[6*20+5]= 3122;       daa[7*20+0]= 665;        daa[7*20+1]= 243;        daa[7*20+2]= 653;        
	daa[7*20+3]= 431;        daa[7*20+4]= 303;        daa[7*20+5]= 133;        daa[7*20+6]= 379;        
	daa[8*20+0]= 66;         daa[8*20+1]= 715;        daa[8*20+2]= 1405;       daa[8*20+3]= 331;        
	daa[8*20+4]= 441;        daa[8*20+5]= 1269;       daa[8*20+6]= 162;        daa[8*20+7]= 19;         
	daa[9*20+0]= 145;        daa[9*20+1]= 136;        daa[9*20+2]= 168;        daa[9*20+3]= 10;         
	daa[9*20+4]= 280;        daa[9*20+5]= 92;         daa[9*20+6]= 148;        daa[9*20+7]= 40;         
	daa[9*20+8]= 29;         daa[10*20+0]= 197;       daa[10*20+1]= 203;       daa[10*20+2]= 113;       
	daa[10*20+3]= 10;        daa[10*20+4]= 396;       daa[10*20+5]= 286;       daa[10*20+6]= 82;        
	daa[10*20+7]= 20;        daa[10*20+8]= 66;        daa[10*20+9]= 1745;      daa[11*20+0]= 236;       
	daa[11*20+1]= 4482;      daa[11*20+2]= 2430;      daa[11*20+3]= 412;       daa[11*20+4]= 48;        
	daa[11*20+5]= 3313;      daa[11*20+6]= 2629;      daa[11*20+7]= 263;       daa[11*20+8]= 305;       
	daa[11*20+9]= 345;       daa[11*20+10]= 218;      daa[12*20+0]= 185;       daa[12*20+1]= 125;       
	daa[12*20+2]= 61;        daa[12*20+3]= 47;        daa[12*20+4]= 159;       daa[12*20+5]= 202;       
	daa[12*20+6]= 113;       daa[12*20+7]= 21;        daa[12*20+8]= 10;        daa[12*20+9]= 1772;      
	daa[12*20+10]= 1351;     daa[12*20+11]= 193;      daa[13*20+0]= 68;        daa[13*20+1]= 53;        
	daa[13*20+2]= 97;        daa[13*20+3]= 22;        daa[13*20+4]= 726;       daa[13*20+5]= 10;        
	daa[13*20+6]= 145;       daa[13*20+7]= 25;        daa[13*20+8]= 127;       daa[13*20+9]= 454;       
	daa[13*20+10]= 1268;     daa[13*20+11]= 72;       daa[13*20+12]= 327;      daa[14*20+0]= 490;       
	daa[14*20+1]= 87;        daa[14*20+2]= 173;       daa[14*20+3]= 170;       daa[14*20+4]= 285;       
	daa[14*20+5]= 323;       daa[14*20+6]= 185;       daa[14*20+7]= 28;        daa[14*20+8]= 152;       
	daa[14*20+9]= 117;       daa[14*20+10]= 219;      daa[14*20+11]= 302;      daa[14*20+12]= 100;      
	daa[14*20+13]= 43;       daa[15*20+0]= 2440;      daa[15*20+1]= 385;       daa[15*20+2]= 2085;      
	daa[15*20+3]= 590;       daa[15*20+4]= 2331;      daa[15*20+5]= 396;       daa[15*20+6]= 568;       
	daa[15*20+7]= 691;       daa[15*20+8]= 303;       daa[15*20+9]= 216;       daa[15*20+10]= 516;      
	daa[15*20+11]= 868;      daa[15*20+12]= 93;       daa[15*20+13]= 487;      daa[15*20+14]= 1202;     
	daa[16*20+0]= 1340;      daa[16*20+1]= 314;       daa[16*20+2]= 1393;      daa[16*20+3]= 266;       
	daa[16*20+4]= 576;       daa[16*20+5]= 241;       daa[16*20+6]= 369;       daa[16*20+7]= 92;        
	daa[16*20+8]= 32;        daa[16*20+9]= 1040;      daa[16*20+10]= 156;      daa[16*20+11]= 918;      
	daa[16*20+12]= 645;      daa[16*20+13]= 148;      daa[16*20+14]= 260;      daa[16*20+15]= 2151;     
	daa[17*20+0]= 14;        daa[17*20+1]= 230;       daa[17*20+2]= 40;        daa[17*20+3]= 18;        
	daa[17*20+4]= 435;       daa[17*20+5]= 53;        daa[17*20+6]= 63;        daa[17*20+7]= 82;        
	daa[17*20+8]= 69;        daa[17*20+9]= 42;        daa[17*20+10]= 159;      daa[17*20+11]= 10;       
	daa[17*20+12]= 86;       daa[17*20+13]= 468;      daa[17*20+14]= 49;       daa[17*20+15]= 73;       
	daa[17*20+16]= 29;       daa[18*20+0]= 56;        daa[18*20+1]= 323;       daa[18*20+2]= 754;       
	daa[18*20+3]= 281;       daa[18*20+4]= 1466;      daa[18*20+5]= 391;       daa[18*20+6]= 142;       
	daa[18*20+7]= 10;        daa[18*20+8]= 1971;      daa[18*20+9]= 89;        daa[18*20+10]= 189;      
	daa[18*20+11]= 247;      daa[18*20+12]= 215;      daa[18*20+13]= 2370;     daa[18*20+14]= 97;       
	daa[18*20+15]= 522;      daa[18*20+16]= 71;       daa[18*20+17]= 346;      daa[19*20+0]= 968;       
	daa[19*20+1]= 92;        daa[19*20+2]= 83;        daa[19*20+3]= 75;        daa[19*20+4]= 592;       
	daa[19*20+5]= 54;        daa[19*20+6]= 200;       daa[19*20+7]= 91;        daa[19*20+8]= 25;        
	daa[19*20+9]= 4797;      daa[19*20+10]= 865;      daa[19*20+11]= 249;      daa[19*20+12]= 475;      
	daa[19*20+13]= 317;      daa[19*20+14]= 122;      daa[19*20+15]= 167;      daa[19*20+16]= 760;      
	daa[19*20+17]= 10;       daa[19*20+18]= 119;      
   
	f[0]= 0.076;            f[1]= 0.062;            f[2]= 0.041;            f[3]= 0.037;            
	f[4]= 0.009;            f[5]= 0.038;            f[6]= 0.049;            f[7]= 0.084;            
	f[8]= 0.025;            f[9]= 0.081;            f[10]= 0.101;           f[11]= 0.05;            
	f[12]= 0.022;           f[13]= 0.051;           f[14]= 0.043;           f[15]= 0.062;           
	f[16]= 0.054;           f[17]= 0.018;           f[18]= 0.031;           f[19]= 0.066; 
      }
      break;
    case VT:
      {
	daa[1*20+0]= 0.233108;   daa[2*20+0]= 0.199097;   daa[2*20+1]= 0.210797;   daa[3*20+0]= 0.265145;   
	daa[3*20+1]= 0.105191;   daa[3*20+2]= 0.883422;   daa[4*20+0]= 0.227333;   daa[4*20+1]= 0.031726;   
	daa[4*20+2]= 0.027495;   daa[4*20+3]= 0.010313;   daa[5*20+0]= 0.310084;   daa[5*20+1]= 0.493763;   
	daa[5*20+2]= 0.2757;     daa[5*20+3]= 0.205842;   daa[5*20+4]= 0.004315;   daa[6*20+0]= 0.567957;   
	daa[6*20+1]= 0.25524;    daa[6*20+2]= 0.270417;   daa[6*20+3]= 1.599461;   daa[6*20+4]= 0.005321;   
	daa[6*20+5]= 0.960976;   daa[7*20+0]= 0.876213;   daa[7*20+1]= 0.156945;   daa[7*20+2]= 0.362028;   
	daa[7*20+3]= 0.311718;   daa[7*20+4]= 0.050876;   daa[7*20+5]= 0.12866;    daa[7*20+6]= 0.250447;   
	daa[8*20+0]= 0.078692;   daa[8*20+1]= 0.213164;   daa[8*20+2]= 0.290006;   daa[8*20+3]= 0.134252;   
	daa[8*20+4]= 0.016695;   daa[8*20+5]= 0.315521;   daa[8*20+6]= 0.104458;   daa[8*20+7]= 0.058131;   
	daa[9*20+0]= 0.222972;   daa[9*20+1]= 0.08151;    daa[9*20+2]= 0.087225;   daa[9*20+3]= 0.01172;    
	daa[9*20+4]= 0.046398;   daa[9*20+5]= 0.054602;   daa[9*20+6]= 0.046589;   daa[9*20+7]= 0.051089;   
	daa[9*20+8]= 0.020039;   daa[10*20+0]= 0.42463;   daa[10*20+1]= 0.192364;  daa[10*20+2]= 0.069245;  
	daa[10*20+3]= 0.060863;  daa[10*20+4]= 0.091709;  daa[10*20+5]= 0.24353;   daa[10*20+6]= 0.151924;  
	daa[10*20+7]= 0.087056;  daa[10*20+8]= 0.103552;  daa[10*20+9]= 2.08989;   daa[11*20+0]= 0.393245;  
	daa[11*20+1]= 1.755838;  daa[11*20+2]= 0.50306;   daa[11*20+3]= 0.261101;  daa[11*20+4]= 0.004067;  
	daa[11*20+5]= 0.738208;  daa[11*20+6]= 0.88863;   daa[11*20+7]= 0.193243;  daa[11*20+8]= 0.153323;  
	daa[11*20+9]= 0.093181;  daa[11*20+10]= 0.201204; daa[12*20+0]= 0.21155;   daa[12*20+1]= 0.08793;   
	daa[12*20+2]= 0.05742;   daa[12*20+3]= 0.012182;  daa[12*20+4]= 0.02369;   daa[12*20+5]= 0.120801;  
	daa[12*20+6]= 0.058643;  daa[12*20+7]= 0.04656;   daa[12*20+8]= 0.021157;  daa[12*20+9]= 0.493845;  
	daa[12*20+10]= 1.105667; daa[12*20+11]= 0.096474; daa[13*20+0]= 0.116646;  daa[13*20+1]= 0.042569;  
	daa[13*20+2]= 0.039769;  daa[13*20+3]= 0.016577;  daa[13*20+4]= 0.051127;  daa[13*20+5]= 0.026235;  
	daa[13*20+6]= 0.028168;  daa[13*20+7]= 0.050143;  daa[13*20+8]= 0.079807;  daa[13*20+9]= 0.32102;   
	daa[13*20+10]= 0.946499; daa[13*20+11]= 0.038261; daa[13*20+12]= 0.173052; daa[14*20+0]= 0.399143;  
	daa[14*20+1]= 0.12848;   daa[14*20+2]= 0.083956;  daa[14*20+3]= 0.160063;  daa[14*20+4]= 0.011137;  
	daa[14*20+5]= 0.15657;   daa[14*20+6]= 0.205134;  daa[14*20+7]= 0.124492;  daa[14*20+8]= 0.078892;  
	daa[14*20+9]= 0.054797;  daa[14*20+10]= 0.169784; daa[14*20+11]= 0.212302; daa[14*20+12]= 0.010363; 
	daa[14*20+13]= 0.042564; daa[15*20+0]= 1.817198;  daa[15*20+1]= 0.292327;  daa[15*20+2]= 0.847049;  
	daa[15*20+3]= 0.461519;  daa[15*20+4]= 0.17527;   daa[15*20+5]= 0.358017;  daa[15*20+6]= 0.406035;  
	daa[15*20+7]= 0.612843;  daa[15*20+8]= 0.167406;  daa[15*20+9]= 0.081567;  daa[15*20+10]= 0.214977; 
	daa[15*20+11]= 0.400072; daa[15*20+12]= 0.090515; daa[15*20+13]= 0.138119; daa[15*20+14]= 0.430431; 
	daa[16*20+0]= 0.877877;  daa[16*20+1]= 0.204109;  daa[16*20+2]= 0.471268;  daa[16*20+3]= 0.178197;  
	daa[16*20+4]= 0.079511;  daa[16*20+5]= 0.248992;  daa[16*20+6]= 0.321028;  daa[16*20+7]= 0.136266;  
	daa[16*20+8]= 0.101117;  daa[16*20+9]= 0.376588;  daa[16*20+10]= 0.243227; daa[16*20+11]= 0.446646; 
	daa[16*20+12]= 0.184609; daa[16*20+13]= 0.08587;  daa[16*20+14]= 0.207143; daa[16*20+15]= 1.767766; 
	daa[17*20+0]= 0.030309;  daa[17*20+1]= 0.046417;  daa[17*20+2]= 0.010459;  daa[17*20+3]= 0.011393;  
	daa[17*20+4]= 0.007732;  daa[17*20+5]= 0.021248;  daa[17*20+6]= 0.018844;  daa[17*20+7]= 0.02399;   
	daa[17*20+8]= 0.020009;  daa[17*20+9]= 0.034954;  daa[17*20+10]= 0.083439; daa[17*20+11]= 0.023321; 
	daa[17*20+12]= 0.022019; daa[17*20+13]= 0.12805;  daa[17*20+14]= 0.014584; daa[17*20+15]= 0.035933; 
	daa[17*20+16]= 0.020437; daa[18*20+0]= 0.087061;  daa[18*20+1]= 0.09701;   daa[18*20+2]= 0.093268;  
	daa[18*20+3]= 0.051664;  daa[18*20+4]= 0.042823;  daa[18*20+5]= 0.062544;  daa[18*20+6]= 0.0552;    
	daa[18*20+7]= 0.037568;  daa[18*20+8]= 0.286027;  daa[18*20+9]= 0.086237;  daa[18*20+10]= 0.189842; 
	daa[18*20+11]= 0.068689; daa[18*20+12]= 0.073223; daa[18*20+13]= 0.898663; daa[18*20+14]= 0.032043; 
	daa[18*20+15]= 0.121979; daa[18*20+16]= 0.094617; daa[18*20+17]= 0.124746; daa[19*20+0]= 1.230985;  
	daa[19*20+1]= 0.113146;  daa[19*20+2]= 0.049824;  daa[19*20+3]= 0.048769;  daa[19*20+4]= 0.163831;  
	daa[19*20+5]= 0.112027;  daa[19*20+6]= 0.205868;  daa[19*20+7]= 0.082579;  daa[19*20+8]= 0.068575;  
	daa[19*20+9]= 3.65443;   daa[19*20+10]= 1.337571; daa[19*20+11]= 0.144587; daa[19*20+12]= 0.307309; 
	daa[19*20+13]= 0.247329; daa[19*20+14]= 0.129315; daa[19*20+15]= 0.1277;   daa[19*20+16]= 0.740372; 
	daa[19*20+17]= 0.022134; daa[19*20+18]= 0.125733; 
	   
	f[0]= 0.078837;         f[1]= 0.051238;         f[2]= 0.042313;         f[3]= 0.053066;         
	f[4]= 0.015175;         f[5]= 0.036713;         f[6]= 0.061924;         f[7]= 0.070852;         
	f[8]= 0.023082;         f[9]= 0.062056;         f[10]= 0.096371;        f[11]= 0.057324;        
	f[12]= 0.023771;        f[13]= 0.043296;        f[14]= 0.043911;        f[15]= 0.063403;        
	f[16]= 0.055897;        f[17]= 0.013272;        f[18]= 0.034399;        f[19]= 0.073101; 
      }
      break;
    case BLOSUM62:
      {
	daa[1*20+0]= 0.735790389698;  daa[2*20+0]= 0.485391055466;  daa[2*20+1]= 1.297446705134;  daa[3*20+0]= 0.543161820899;  
	daa[3*20+1]= 0.500964408555;  daa[3*20+2]= 3.180100048216;  daa[4*20+0]= 1.45999531047;   daa[4*20+1]= 0.227826574209;  
	daa[4*20+2]= 0.397358949897;  daa[4*20+3]= 0.240836614802;  daa[5*20+0]= 1.199705704602;  daa[5*20+1]= 3.020833610064;  
	daa[5*20+2]= 1.839216146992;  daa[5*20+3]= 1.190945703396;  daa[5*20+4]= 0.32980150463;   daa[6*20+0]= 1.1709490428;    
	daa[6*20+1]= 1.36057419042;   daa[6*20+2]= 1.24048850864;   daa[6*20+3]= 3.761625208368;  daa[6*20+4]= 0.140748891814;  
	daa[6*20+5]= 5.528919177928;  daa[7*20+0]= 1.95588357496;   daa[7*20+1]= 0.418763308518;  daa[7*20+2]= 1.355872344485;  
	daa[7*20+3]= 0.798473248968;  daa[7*20+4]= 0.418203192284;  daa[7*20+5]= 0.609846305383;  daa[7*20+6]= 0.423579992176;  
	daa[8*20+0]= 0.716241444998;  daa[8*20+1]= 1.456141166336;  daa[8*20+2]= 2.414501434208;  daa[8*20+3]= 0.778142664022;  
	daa[8*20+4]= 0.354058109831;  daa[8*20+5]= 2.43534113114;   daa[8*20+6]= 1.626891056982;  daa[8*20+7]= 0.539859124954;  
	daa[9*20+0]= 0.605899003687;  daa[9*20+1]= 0.232036445142;  daa[9*20+2]= 0.283017326278;  daa[9*20+3]= 0.418555732462;  
	daa[9*20+4]= 0.774894022794;  daa[9*20+5]= 0.236202451204;  daa[9*20+6]= 0.186848046932;  daa[9*20+7]= 0.189296292376;  
	daa[9*20+8]= 0.252718447885;  daa[10*20+0]= 0.800016530518; daa[10*20+1]= 0.622711669692; daa[10*20+2]= 0.211888159615; 
	daa[10*20+3]= 0.218131577594; daa[10*20+4]= 0.831842640142; daa[10*20+5]= 0.580737093181; daa[10*20+6]= 0.372625175087; 
	daa[10*20+7]= 0.217721159236; daa[10*20+8]= 0.348072209797; daa[10*20+9]= 3.890963773304; daa[11*20+0]= 1.295201266783; 
	daa[11*20+1]= 5.411115141489; daa[11*20+2]= 1.593137043457; daa[11*20+3]= 1.032447924952; daa[11*20+4]= 0.285078800906; 
	daa[11*20+5]= 3.945277674515; daa[11*20+6]= 2.802427151679; daa[11*20+7]= 0.752042440303; daa[11*20+8]= 1.022507035889; 
	daa[11*20+9]= 0.406193586642; daa[11*20+10]= 0.445570274261;daa[12*20+0]= 1.253758266664; daa[12*20+1]= 0.983692987457; 
	daa[12*20+2]= 0.648441278787; daa[12*20+3]= 0.222621897958; daa[12*20+4]= 0.76768882348;  daa[12*20+5]= 2.494896077113; 
	daa[12*20+6]= 0.55541539747;  daa[12*20+7]= 0.459436173579; daa[12*20+8]= 0.984311525359; daa[12*20+9]= 3.364797763104; 
	daa[12*20+10]= 6.030559379572;daa[12*20+11]= 1.073061184332;daa[13*20+0]= 0.492964679748; daa[13*20+1]= 0.371644693209; 
	daa[13*20+2]= 0.354861249223; daa[13*20+3]= 0.281730694207; daa[13*20+4]= 0.441337471187; daa[13*20+5]= 0.14435695975;  
	daa[13*20+6]= 0.291409084165; daa[13*20+7]= 0.368166464453; daa[13*20+8]= 0.714533703928; daa[13*20+9]= 1.517359325954; 
	daa[13*20+10]= 2.064839703237;daa[13*20+11]= 0.266924750511;daa[13*20+12]= 1.77385516883; daa[14*20+0]= 1.173275900924; 
	daa[14*20+1]= 0.448133661718; daa[14*20+2]= 0.494887043702; daa[14*20+3]= 0.730628272998; daa[14*20+4]= 0.356008498769; 
	daa[14*20+5]= 0.858570575674; daa[14*20+6]= 0.926563934846; daa[14*20+7]= 0.504086599527; daa[14*20+8]= 0.527007339151; 
	daa[14*20+9]= 0.388355409206; daa[14*20+10]= 0.374555687471;daa[14*20+11]= 1.047383450722;daa[14*20+12]= 0.454123625103;
	daa[14*20+13]= 0.233597909629;daa[15*20+0]= 4.325092687057; daa[15*20+1]= 1.12278310421;  daa[15*20+2]= 2.904101656456; 
	daa[15*20+3]= 1.582754142065; daa[15*20+4]= 1.197188415094; daa[15*20+5]= 1.934870924596; daa[15*20+6]= 1.769893238937; 
	daa[15*20+7]= 1.509326253224; daa[15*20+8]= 1.11702976291;  daa[15*20+9]= 0.35754441246;  daa[15*20+10]= 0.352969184527;
	daa[15*20+11]= 1.752165917819;daa[15*20+12]= 0.918723415746;daa[15*20+13]= 0.540027644824;daa[15*20+14]= 1.169129577716;
	daa[16*20+0]= 1.729178019485; daa[16*20+1]= 0.914665954563; daa[16*20+2]= 1.898173634533; daa[16*20+3]= 0.934187509431; 
	daa[16*20+4]= 1.119831358516; daa[16*20+5]= 1.277480294596; daa[16*20+6]= 1.071097236007; daa[16*20+7]= 0.641436011405; 
	daa[16*20+8]= 0.585407090225; daa[16*20+9]= 1.17909119726;  daa[16*20+10]= 0.915259857694;daa[16*20+11]= 1.303875200799;
	daa[16*20+12]= 1.488548053722;daa[16*20+13]= 0.488206118793;daa[16*20+14]= 1.005451683149;daa[16*20+15]= 5.15155629227; 
	daa[17*20+0]= 0.465839367725; daa[17*20+1]= 0.426382310122; daa[17*20+2]= 0.191482046247; daa[17*20+3]= 0.145345046279; 
	daa[17*20+4]= 0.527664418872; daa[17*20+5]= 0.758653808642; daa[17*20+6]= 0.407635648938; daa[17*20+7]= 0.508358924638; 
	daa[17*20+8]= 0.30124860078;  daa[17*20+9]= 0.34198578754;  daa[17*20+10]= 0.6914746346;  daa[17*20+11]= 0.332243040634;
	daa[17*20+12]= 0.888101098152;daa[17*20+13]= 2.074324893497;daa[17*20+14]= 0.252214830027;daa[17*20+15]= 0.387925622098;
	daa[17*20+16]= 0.513128126891;daa[18*20+0]= 0.718206697586; daa[18*20+1]= 0.720517441216; daa[18*20+2]= 0.538222519037; 
	daa[18*20+3]= 0.261422208965; daa[18*20+4]= 0.470237733696; daa[18*20+5]= 0.95898974285;  daa[18*20+6]= 0.596719300346; 
	daa[18*20+7]= 0.308055737035; daa[18*20+8]= 4.218953969389; daa[18*20+9]= 0.674617093228; daa[18*20+10]= 0.811245856323;
	daa[18*20+11]= 0.7179934869;  daa[18*20+12]= 0.951682162246;daa[18*20+13]= 6.747260430801;daa[18*20+14]= 0.369405319355;
	daa[18*20+15]= 0.796751520761;daa[18*20+16]= 0.801010243199;daa[18*20+17]= 4.054419006558;daa[19*20+0]= 2.187774522005; 
	daa[19*20+1]= 0.438388343772; daa[19*20+2]= 0.312858797993; daa[19*20+3]= 0.258129289418; daa[19*20+4]= 1.116352478606; 
	daa[19*20+5]= 0.530785790125; daa[19*20+6]= 0.524253846338; daa[19*20+7]= 0.25334079019;  daa[19*20+8]= 0.20155597175;  
	daa[19*20+9]= 8.311839405458; daa[19*20+10]= 2.231405688913;daa[19*20+11]= 0.498138475304;daa[19*20+12]= 2.575850755315;
	daa[19*20+13]= 0.838119610178;daa[19*20+14]= 0.496908410676;daa[19*20+15]= 0.561925457442;daa[19*20+16]= 2.253074051176;
	daa[19*20+17]= 0.266508731426;daa[19*20+18]= 1;             
	   
	f[0]= 0.074;                 f[1]= 0.052;                 f[2]= 0.045;                 f[3]= 0.054;                 
	f[4]= 0.025;                 f[5]= 0.034;                 f[6]= 0.054;                 f[7]= 0.074;                 
	f[8]= 0.026;                 f[9]= 0.068;                 f[10]= 0.099;                f[11]= 0.058;                
	f[12]= 0.025;                f[13]= 0.047;                f[14]= 0.039;                f[15]= 0.057;                
	f[16]= 0.051;                f[17]= 0.013;                f[18]= 0.032;                f[19]= 0.073; 
      }
      break;
    case MTMAM:
      {
	daa[1*20+0]= 32;              daa[2*20+0]= 2;    daa[2*20+1]= 4;               daa[3*20+0]= 11;
	daa[3*20+1]= 0;               daa[3*20+2]= 864;  daa[4*20+0]= 0;               daa[4*20+1]= 186;
	daa[4*20+2]= 0;               daa[4*20+3]= 0;    daa[5*20+0]= 0;               daa[5*20+1]= 246;
	daa[5*20+2]= 8;               daa[5*20+3]= 49;   daa[5*20+4]= 0;               daa[6*20+0]= 0;
	daa[6*20+1]= 0;               daa[6*20+2]= 0;    daa[6*20+3]= 569;             daa[6*20+4]= 0;
	daa[6*20+5]= 274;             daa[7*20+0]= 78;   daa[7*20+1]= 18;              daa[7*20+2]= 47;
	daa[7*20+3]= 79;              daa[7*20+4]= 0;    daa[7*20+5]= 0;               daa[7*20+6]= 22;
	daa[8*20+0]= 8;               daa[8*20+1]= 232;  daa[8*20+2]= 458;             daa[8*20+3]= 11;
	daa[8*20+4]= 305;             daa[8*20+5]= 550;  daa[8*20+6]= 22;              daa[8*20+7]= 0;
	daa[9*20+0]= 75;              daa[9*20+1]= 0;    daa[9*20+2]= 19;              daa[9*20+3]= 0;
	daa[9*20+4]= 41;              daa[9*20+5]= 0;    daa[9*20+6]= 0;               daa[9*20+7]= 0;
	daa[9*20+8]= 0;               daa[10*20+0]= 21;  daa[10*20+1]= 6;              daa[10*20+2]= 0;
	daa[10*20+3]= 0;              daa[10*20+4]= 27;  daa[10*20+5]= 20;             daa[10*20+6]= 0;
	daa[10*20+7]= 0;              daa[10*20+8]= 26;  daa[10*20+9]= 232;            daa[11*20+0]= 0;
	daa[11*20+1]= 50;             daa[11*20+2]= 408; daa[11*20+3]= 0;              daa[11*20+4]= 0;
	daa[11*20+5]= 242;            daa[11*20+6]= 215; daa[11*20+7]= 0;              daa[11*20+8]= 0;
	daa[11*20+9]= 6;              daa[11*20+10]= 4;  daa[12*20+0]= 76;             daa[12*20+1]= 0;
	daa[12*20+2]= 21;             daa[12*20+3]= 0;   daa[12*20+4]= 0;              daa[12*20+5]= 22;
	daa[12*20+6]= 0;              daa[12*20+7]= 0;   daa[12*20+8]= 0;              daa[12*20+9]= 378;
	daa[12*20+10]= 609;           daa[12*20+11]= 59; daa[13*20+0]= 0;              daa[13*20+1]= 0;
	daa[13*20+2]= 6;              daa[13*20+3]= 5;   daa[13*20+4]= 7;              daa[13*20+5]= 0;
	daa[13*20+6]= 0;              daa[13*20+7]= 0;   daa[13*20+8]= 0;              daa[13*20+9]= 57;
	daa[13*20+10]= 246;           daa[13*20+11]= 0;  daa[13*20+12]= 11;            daa[14*20+0]= 53;
	daa[14*20+1]= 9;              daa[14*20+2]= 33;  daa[14*20+3]= 2;              daa[14*20+4]= 0;
	daa[14*20+5]= 51;             daa[14*20+6]= 0;   daa[14*20+7]= 0;              daa[14*20+8]= 53;
	daa[14*20+9]= 5;              daa[14*20+10]= 43; daa[14*20+11]= 18;            daa[14*20+12]= 0;
	daa[14*20+13]= 17;            daa[15*20+0]= 342; daa[15*20+1]= 3;              daa[15*20+2]= 446;
	daa[15*20+3]= 16;             daa[15*20+4]= 347; daa[15*20+5]= 30;             daa[15*20+6]= 21;
	daa[15*20+7]= 112;            daa[15*20+8]= 20;  daa[15*20+9]= 0;              daa[15*20+10]= 74;
	daa[15*20+11]= 65;            daa[15*20+12]= 47; daa[15*20+13]= 90;            daa[15*20+14]= 202;
	daa[16*20+0]= 681;            daa[16*20+1]= 0;   daa[16*20+2]= 110;            daa[16*20+3]= 0;
	daa[16*20+4]= 114;            daa[16*20+5]= 0;   daa[16*20+6]= 4;              daa[16*20+7]= 0;
	daa[16*20+8]= 1;              daa[16*20+9]= 360; daa[16*20+10]= 34;            daa[16*20+11]= 50;
	daa[16*20+12]= 691;           daa[16*20+13]= 8;  daa[16*20+14]= 78;            daa[16*20+15]= 614;
	daa[17*20+0]= 5;              daa[17*20+1]= 16;  daa[17*20+2]= 6;              daa[17*20+3]= 0;
	daa[17*20+4]= 65;             daa[17*20+5]= 0;   daa[17*20+6]= 0;              daa[17*20+7]= 0;
	daa[17*20+8]= 0;              daa[17*20+9]= 0;   daa[17*20+10]= 12;            daa[17*20+11]= 0;
	daa[17*20+12]= 13;            daa[17*20+13]= 0;  daa[17*20+14]= 7;             daa[17*20+15]= 17;
	daa[17*20+16]= 0;             daa[18*20+0]= 0;   daa[18*20+1]= 0;              daa[18*20+2]= 156;
	daa[18*20+3]= 0;              daa[18*20+4]= 530; daa[18*20+5]= 54;             daa[18*20+6]= 0;
	daa[18*20+7]= 1;              daa[18*20+8]= 1525;daa[18*20+9]= 16;             daa[18*20+10]= 25;
	daa[18*20+11]= 67;            daa[18*20+12]= 0;  daa[18*20+13]= 682;           daa[18*20+14]= 8;
	daa[18*20+15]= 107;           daa[18*20+16]= 0;  daa[18*20+17]= 14;            daa[19*20+0]= 398;
	daa[19*20+1]= 0;              daa[19*20+2]= 0;   daa[19*20+3]= 10;             daa[19*20+4]= 0;
	daa[19*20+5]= 33;             daa[19*20+6]= 20;  daa[19*20+7]= 5;              daa[19*20+8]= 0;
	daa[19*20+9]= 2220;           daa[19*20+10]= 100;daa[19*20+11]= 0;             daa[19*20+12]= 832;
	daa[19*20+13]= 6;             daa[19*20+14]= 0;  daa[19*20+15]= 0;             daa[19*20+16]= 237;
	daa[19*20+17]= 0;             daa[19*20+18]= 0;       
    
	f[0]= 0.0692;  f[1]=  0.0184;  f[2]= 0.04;    f[3]= 0.0186;
	f[4]= 0.0065;  f[5]=  0.0238;  f[6]= 0.0236;  f[7]= 0.0557;
	f[8]= 0.0277;  f[9]=  0.0905;  f[10]=0.1675;  f[11]= 0.0221;
	f[12]=0.0561;  f[13]= 0.0611;  f[14]=0.0536;  f[15]= 0.0725;
	f[16]=0.087;   f[17]= 0.0293;  f[18]=0.034;   f[19]= 0.0428;
      }
      break;
    case GTR:
      printf("this function should not be called by GTR model for proteins\n");
      exit(-1);
    default: 
      printf("FATAL ERROR not defined\n");
      exit(-1);
    }

  for (i=0; i<20; i++)  
    for (j=0; j<i; j++)  
      daa[j*20+i] = daa[i*20+j];

  max = 0;
  
  for(i = 0; i < 19; i++)
    for(j = i + 1; j < 20; j++)
      {
	q[i][j] = temp = daa[i * 20 + j];
	if(temp > max) 
	  max = temp;
      }
 
  scaler = AA_SCALE / max;
   
  /* SCALING HAS BEEN RE-INTRODUCED TO RESOLVE NUMERICAL  PROBLEMS */   

  r = 0;
  for(i = 0; i < 19; i++)
    {      
      for(j = i + 1; j < 20; j++)
	{ 
	  q[i][j] *= scaler;

	  assert(q[i][j] <= AA_SCALE_PLUS_EPSILON);
	  
	  initialRates[r++] = q[i][j];
	}
    }             
}

void initReversibleGTR(tree *tr, analdef *adef, int model)
{ 
  if(adef->model == M_PROTCAT || adef->model == M_PROTGAMMA)
    {
      double r[20][20], a[20][20], *initialRates = &(tr->initialRates[model * 190]), 
	f[20], e[20], d[20];
      int i, j, k, m, l;
      double invfreq[20], EIGN[20], EIGV[20][20]; 
      double *eptr;         
      
      if(adef->useMultipleModel)
	{
	  if(adef->proteinMatrix == GTR)
	    {
	      printf("FATAL ERROR no GTR for AA with multiple models\n");
	      exit(-1);
	    }
	  else
	    {	     
	      initProtMat(tr, model, f, adef->protModels[model]);
	      
	      if(adef->protFreqs[model])
		{	     	      
		  for(l = 0; l < 20; l++)		
		    f[l] = tr->frequencies[model * 20 + l];			       
		}
	      else
		{	     
		  for(l = 0; l < 20; l++)		
		    tr->frequencies[model * 20 + l] = f[l];
		}
	    }
	}
      else
	{
	  if(adef->proteinMatrix == GTR)
	    {
	      for(l = 0; l < 20; l++)
		f[l] = tr->frequencies[model * 20 + l];
	    }
	  else
	    {	  
	      initProtMat(tr, model, f, adef->proteinMatrix);
	      if(adef->protEmpiricalFreqs)
		{	     	      
		  for(l = 0; l < 20; l++)		
		    f[l] = tr->frequencies[model * 20 + l];			       
		}
	      else
		{	     
		  for(l = 0; l < 20; l++)		
		    tr->frequencies[model * 20 + l] = f[l];
		}
	    }
	}
                 
      i = 0;
      
      for(j = 0; j < 19; j++)
	for (k = j+1; k < 20; k++)      	  
	  r[j][k] = initialRates[i++];     
            
      for (j = 0; j < 20; j++) 
	{
	  r[j][j] = 0.0;
	  for (k = 0; k < j; k++)
	    r[j][k] = r[k][j];
	}           
     
      /* TODO MULT SOSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSS*/

      tr->fracchanges[model] = 0;
      tr->fracchange = 0;

      for (j = 0; j< 20; j++)
	for (k = 0; k< 20; k++)
	  tr->fracchanges[model] += f[j] * r[j][k] * f[k];
      
      if(tr->NumberOfModels == 1)    
	tr->fracchange = tr->fracchanges[model];
         
      /*TODO MULT SOSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSS*/

      m = 0;
      
      for(i=0; i< 20; i++) 
	a[i][i] = 0;
      
      for(i=0; i < 20; i++) 
	{
	  for(j=i+1;  j < 20 ; j++) 
	    {
	      double factor =  initialRates[m++];
	      a[i][j] = a[j][i] = factor * sqrt( f[i] * f[j]);
	      a[i][i] -= factor * f[j];
	      a[j][j] -= factor * f[i];
	    }
	}
           
      tred2((double *)a,20,20,d,e);       
      tqli(d, e, 20 , 20, (double *)a);   
      
      for(i=0; i<20; i++)     
	for(j=0; j<20; j++)       
	  a[i][j] *= sqrt(f[j]);
      
      for (i=0; i<20; i++)
	{	  
	  if (d[i] > -1e-8) 
	    {	      
	      if (i != 0) 
		{		    
		  double tmp = d[i], sum=0;
		  d[i] = d[0];
		  d[0] = tmp;
		  for (j=0; j < 20; j++) 
		    {
		      tmp = a[i][j];
		      a[i][j] = a[0][j];
		      sum += (a[0][j] = tmp);
		    }
		  for (j=0; j < 20; j++) 
		    a[0][j] /= sum;
		}
	      break;
	    }
	}
  
      for (i=0; i< 20; i++) 
	{
	  EIGN[i] = -d[i];
	  
	  for (j=0; j<20; j++)
	    EIGV[i][j] = a[j][i];
	  invfreq[i] = 1 / EIGV[i][0]; 
	}                                    

      for(l = 1; l < 20; l++)
	tr->EIGN[model * 19 + (l - 1)] = EIGN[l]; 
      
      eptr = &(tr->EV[model * 400]);
      
      for(i = 0; i < 20; i++)		  
	for(j = 0; j < 20; j++)      	    
	  *eptr++ = EIGV[i][j];	             	    	     

      for(i = 0; i < 20; i++)
	for(j = 1; j < 20; j++)
	  tr->EI[model * 380 + i * 19 + (j - 1)] = tr->EV[model * 400 + i * 20 + j] * invfreq[i];        	             
    
      for(i=0; i < 23; i++)
	{     
	  for(j = 0; j < 20; j++)
	    {
	      tr->tipVector[model * 460 + 20 * i + j] = 0.0; 	     
	    }
	  
	  if(i < 20)
	    {	  
	      for (j = 0; j < 20; j++) 	    
		{
		  tr->tipVector[model * 460 + 20 * i + j] += EIGV[i][j];	 	       
		}
	    }
	  else
	    {
	      if(i == 20)
		{
		  for (j = 0; j < 20; j++) 	    
		    {
		      tr->tipVector[model * 460 + 20 * i + j] += EIGV[2][j] + EIGV[3][j];		    
		    }
		}
	      else
		{
		  if(i == 21)
		    {
		      for (j = 0; j < 20; j++) 	    
			{
			  tr->tipVector[model * 460 + 20 * i + j] += EIGV[5][j] + EIGV[6][j];			
			}
		    }
		  else
		    {
		      if(i == 22)
			{
			  for (j = 0; j < 20; j++) 	    
			    {
			      for(k = 0; k < 20; k++)
				{
				  tr->tipVector[model * 460 + 20 * i + j] += EIGV[k][j];			       
				}
			    }
			}
		    }
		}
	    }
	}
    }
  else
    {
      double r[4][4], a[4][4], *initialRates = &(tr->initialRates[model * 5]), 
	f[4], e[4], d[4];
      int i, j, k, m, code;
      double invfreq[4], EIGN[4], EIGV[4][4]; 
      double *eptr;
    
      f[0] = tr->frequencies[model * 4];
      f[1] = tr->frequencies[model * 4 + 1];
      f[2] = tr->frequencies[model * 4 + 2];
      f[3] = tr->frequencies[model * 4 + 3];
      
      i = 0;
      for (j = 0; j < 2; j++)
	for (k = j+1; k< 4; k++)	  
	  r[j][k] = initialRates[i++];
      r[2][3] = 1.0;
      for (j = 0; j < 4; j++) 
	{
	  r[j][j] = 0;
	  for (k = 0; k < j; k++)
	    r[j][k] = r[k][j];
	}
      
      
      /* TODO MULT SOSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSS*/
      tr->fracchanges[model] = 0;
      tr->fracchange = 0;
      
      for (j = 0; j<4; j++)
	for (k = 0; k<4; k++)
	  tr->fracchanges[model] += f[j] * r[j][k] * f[k];
      
      if(tr->NumberOfModels == 1)    
	tr->fracchange = tr->fracchanges[model];
      
      /*TODO MULT SOSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSS*/
      m = 0;
      
      for(i=0; i<4; i++) 
	a[i][i] = 0;            

      for(i=0; i<4; i++) 
	{
	  for(j=i+1; j<4; j++) 
	    {
	      double factor = ((m>=5) ? 1.0 : initialRates[m++]);
	      a[i][j] = a[j][i] = factor * sqrt(f[i] * f[j]);
	      a[i][i] -= factor * f[j];
	      a[j][j] -= factor * f[i];	     
	    }
	}           
     
     
      tred2((double *)a,4,4,d,e);       
      tqli(d, e, 4 , 4, (double *)a);
     
     

      for(i=0; i<4; i++)     
	for(j=0; j<4; j++)       
	  a[i][j] *= sqrt(f[j]);
      
      for (i=0; i<4; i++)
	{	  
	  if (d[i] > -1e-8) 
	    {	      
	      if (i != 0)
		{		    
		  double tmp = d[i], sum=0;
		  d[i] = d[0];
		  d[0] = tmp;
		  for (j=0; j < 4; j++) 
		    {
		      tmp = a[i][j];
		      a[i][j] = a[0][j];
		      sum += (a[0][j] = tmp);
		    }
		  for (j=0; j < 4; j++) 
		    a[0][j] /= sum;
		}
	      break;
	    }
	}
  
      for (i=0; i< 4; i++) 
	{
	  EIGN[i] = -d[i];
	  
	  for (j=0; j<4; j++)
	    EIGV[i][j] = a[j][i];
	  invfreq[i] = 1 / EIGV[i][0]; 
	}                   
                 
      tr->EIGN[model * 3] = EIGN[1];
      tr->EIGN[model * 3 + 1] = EIGN[2];
      tr->EIGN[model * 3 + 2] = EIGN[3];        

      eptr = &(tr->EV[model * 16]);

      for(i = 0; i < 4; i++)       
	for(j = 0; j < 4; j++) 	  	  
	  *eptr++ = EIGV[i][j];	    	   
      
      tr->EI[model * 12]     = tr->EV[model * 16 + 1] * invfreq[0];
      tr->EI[model * 12 + 1] = tr->EV[model * 16 + 2] * invfreq[0];
      tr->EI[model * 12 + 2] = tr->EV[model * 16 + 3] * invfreq[0];
      
      tr->EI[model * 12 + 3] = tr->EV[model * 16 + 5] * invfreq[1];
      tr->EI[model * 12 + 4] = tr->EV[model * 16 + 6] * invfreq[1];
      tr->EI[model * 12 + 5] = tr->EV[model * 16 + 7] * invfreq[1];
      
      tr->EI[model * 12 + 6] = tr->EV[model * 16 + 9]  * invfreq[2];
      tr->EI[model * 12 + 7] = tr->EV[model * 16 + 10] * invfreq[2];
      tr->EI[model * 12 + 8] = tr->EV[model * 16 + 11] * invfreq[2];
      
      tr->EI[model * 12 + 9]  = tr->EV[model * 16 + 13] * invfreq[3];
      tr->EI[model * 12 + 10] = tr->EV[model * 16 + 14] * invfreq[3];
      tr->EI[model * 12 + 11] = tr->EV[model * 16 + 15] * invfreq[3];
           
      for(i=0; i < 16; i++)
	{
	  code = i;

	  tr->tipVector[model * 64 + i * 4]     = 0;
	  tr->tipVector[model * 64 + i * 4 + 1] = 0;
	  tr->tipVector[model * 64 + i * 4 + 2] = 0;
	  tr->tipVector[model * 64 + i * 4 + 3] = 0;	 

	  if(i > 0)
	    {		      
	      for (j = 0; j < 4; j++) 
		{	    
		  if ((code >> j) & 1) 
		    {
		      int jj = "0123"[j] - '0';				 		     
		      tr->tipVector[model * 64 + i * 4]     += EIGV[jj][0];
		      tr->tipVector[model * 64 + i * 4 + 1] += EIGV[jj][1];
		      tr->tipVector[model * 64 + i * 4 + 2] += EIGV[jj][2];
		      tr->tipVector[model * 64 + i * 4 + 3] += EIGV[jj][3];		      		     		      
		    }	     		  
		}	    
	    }     
	}
    }
  
}


double LnGamma (double alpha)
{
/* returns ln(gamma(alpha)) for alpha>0, accurate to 10 decimal places.  
   Stirling's formula is used for the central polynomial part of the procedure.
   Pike MC & Hill ID (1966) Algorithm 291: Logarithm of the gamma function.
   Communications of the Association for Computing Machinery, 9:684
*/
  double x, f, z, result;

  x = alpha;
  f = 0.0;
  
  if ( x < 7.0) 
     {
       f = 1.0;  
       z = alpha - 1.0;
      
       while ((z = z + 1.0) < 7.0)  
	 {	  
	   f *= z;
	 }
       x = z;   
     
       assert(f != 0.0);
	
       f=-log(f);
     }
   
   z = 1/(x*x);
   
   result = f + (x-0.5)*log(x) - x + .918938533204673 
	  + (((-.000595238095238*z+.000793650793651)*z-.002777777777778)*z
	       +.083333333333333)/x;  

   return result;
}



double IncompleteGamma (double x, double alpha, double ln_gamma_alpha)
{
/* returns the incomplete gamma ratio I(x,alpha) where x is the upper 
	   limit of the integration and alpha is the shape parameter.
   returns (-1) if in error
   ln_gamma_alpha = ln(Gamma(alpha)), is almost redundant.
   (1) series expansion     if (alpha>x || x<=1)
   (2) continued fraction   otherwise
   RATNEST FORTRAN by
   Bhattacharjee GP (1970) The incomplete gamma integral.  Applied Statistics,
   19: 285-287 (AS32)
*/
   int i;
   double p=alpha, g=ln_gamma_alpha;
   double accurate=1e-8, overflow=1e30;
   double factor, gin=0, rn=0, a=0,b=0,an=0,dif=0, term=0, pn[6];


   if (x==0) return (0);
   if (x<0 || p<=0) return (-1);

   
   factor=exp(p*log(x)-x-g);   
   if (x>1 && x>=p) goto l30;
   /* (1) series expansion */
   gin=1;  term=1;  rn=p;
 l20:
   rn++;
   term*=x/rn;   gin+=term;

   if (term > accurate) goto l20;
   gin*=factor/p;
   goto l50;
 l30:  
   /* (2) continued fraction */
   a=1-p;   b=a+x+1;  term=0;
   pn[0]=1;  pn[1]=x;  pn[2]=x+1;  pn[3]=x*b;
   gin=pn[2]/pn[3];   
 l32:  
   a++;  
   b+=2;  
   term++;   
   an=a*term;
   for (i=0; i<2; i++) 
     pn[i+4]=b*pn[i+2]-an*pn[i];
   if (pn[5] == 0) goto l35;
   rn=pn[4]/pn[5];   
   dif=fabs(gin-rn);  
   if (dif>accurate) goto l34;
   if (dif<=accurate*rn) goto l42;
 l34:   
   gin=rn;
 l35: 
   for (i=0; i<4; i++) 
     pn[i]=pn[i+2];
   if (fabs(pn[4]) < overflow)            
     goto l32;        
   
   for (i=0; i<4; i++) 
     pn[i]/=overflow;

   
   goto l32;
 l42:  
   gin=1-factor*gin;

 l50: 
   return (gin);
}




double PointNormal (double prob)
{
/* returns z so that Prob{x<z}=prob where x ~ N(0,1) and (1e-12)<prob<1-(1e-12)
   returns (-9999) if in error
   Odeh RE & Evans JO (1974) The percentage points of the normal distribution.
   Applied Statistics 22: 96-97 (AS70)

   Newer methods:
     Wichura MJ (1988) Algorithm AS 241: the percentage points of the
       normal distribution.  37: 477-484.
     Beasley JD & Springer SG  (1977).  Algorithm AS 111: the percentage 
       points of the normal distribution.  26: 118-121.

*/
   double a0=-.322232431088, a1=-1, a2=-.342242088547, a3=-.0204231210245;
   double a4=-.453642210148e-4, b0=.0993484626060, b1=.588581570495;
   double b2=.531103462366, b3=.103537752850, b4=.0038560700634;
   double y, z=0, p=prob, p1;

   p1 = (p<0.5 ? p : 1-p);
   if (p1<1e-20) return (-9999);

   y = sqrt (log(1/(p1*p1)));   
   z = y + ((((y*a4+a3)*y+a2)*y+a1)*y+a0) / ((((y*b4+b3)*y+b2)*y+b1)*y+b0);
   return (p<0.5 ? -z : z);
}


double PointChi2 (double prob, double v)
{
/* returns z so that Prob{x<z}=prob where x is Chi2 distributed with df=v
   returns -1 if in error.   0.000002<prob<0.999998
   RATNEST FORTRAN by
       Best DJ & Roberts DE (1975) The percentage points of the 
       Chi2 distribution.  Applied Statistics 24: 385-388.  (AS91)
   Converted into C by Ziheng Yang, Oct. 1993.
*/
   double e=.5e-6, aa=.6931471805, p=prob, g;
   double xx, c, ch, a=0,q=0,p1=0,p2=0,t=0,x=0,b=0,s1,s2,s3,s4,s5,s6;
  
   if (p<.000002 || p>.999998 || v<=0) return (-1);
  
   g = LnGamma(v/2);
   
   xx=v/2;   c=xx-1;
   if (v >= -1.24*log(p)) goto l1;

   ch=pow((p*xx*exp(g+xx*aa)), 1/xx);
   if (ch-e<0) return (ch);
   goto l4;
l1:
   if (v>.32) goto l3;
   ch=0.4;   a=log(1-p);
l2:
   q=ch;  p1=1+ch*(4.67+ch);  p2=ch*(6.73+ch*(6.66+ch));
   t=-0.5+(4.67+2*ch)/p1 - (6.73+ch*(13.32+3*ch))/p2;
   ch-=(1-exp(a+g+.5*ch+c*aa)*p2/p1)/t;
   if (fabs(q/ch-1)-.01 <= 0) goto l4;
   else                       goto l2;
  
l3:    
   x=PointNormal (p);
   p1=0.222222/v;   ch=v*pow((x*sqrt(p1)+1-p1), 3.0);
   if (ch>2.2*v+6)  ch=-2*(log(1-p)-c*log(.5*ch)+g);
l4:
   q=ch;   p1=.5*ch;   
   if ((t=IncompleteGamma (p1, xx, g))< 0.0) 
     {
       printf ("IncompleteGamma \n");      
       return (-1);
     }
  
   p2=p-t;
   t=p2*exp(xx*aa+g+p1-c*log(ch));   
   b=t/ch;  a=0.5*t-b*c;

   s1=(210+a*(140+a*(105+a*(84+a*(70+60*a))))) / 420;
   s2=(420+a*(735+a*(966+a*(1141+1278*a))))/2520;
   s3=(210+a*(462+a*(707+932*a)))/2520;
   s4=(252+a*(672+1182*a)+c*(294+a*(889+1740*a)))/5040;
   s5=(84+264*a+c*(175+606*a))/2520;
   s6=(120+c*(346+127*c))/5040;
   ch+=t*(1+0.5*t*s1-b*c*(s1-b*(s2-b*(s3-b*(s4-b*(s5-b*s6))))));
   if (fabs(q/ch-1) > e) goto l4;

   return (ch);
}






void makeGammaCats(tree *tr, int model)
{
  int i, K = tr->gammaCategories;
  double factor, lnga1, alfa, beta;
  double *gammaProbs = (double *)malloc(K * sizeof(double));

  alfa = beta = tr->alphas[model];

  assert(alfa >= ALPHA_MIN);  
 
  factor = alfa / beta * K;  

  assert(K != 1); 
    
  lnga1=LnGamma(alfa+1);
   
  for (i=0; i<K-1; i++)
    gammaProbs[i]=PointGamma((i+1.0)/K, alfa, beta);
  
  for (i=0; i<K-1; i++)
    gammaProbs[i]=IncompleteGamma(gammaProbs[i]*beta, alfa+1, lnga1);   
  
  tr->gammaRates[model * K] = gammaProbs[0]*factor;
  
  tr->gammaRates[model * K + (K-1)] = (1 - gammaProbs[K-2])*factor;
  
  for (i=1; i<K-1; i++)  
    tr->gammaRates[model * K + i] = (gammaProbs[i] - gammaProbs[i-1])*factor;      
    
  free(gammaProbs);

  return;  
}

void assignLikelihoodFunctions(analdef *adef)
{
  switch(adef->model)
    {       
    case M_PROTGAMMA:           
       if(adef->useMultipleModel)
	{	  
	  newview = newviewGTRGAMMAPROTMULT;
	  newviewPartition = newviewGTRGAMMAPROTMULTPARTITION;
	  
	  if(adef->useInvariant)
	    {
	      makenewz          = makenewzGTRGAMMAPROTMULTINVAR;
	      makenewzPartition = makenewzGTRGAMMAPROTMULTINVARPARTITION;
	      evaluate          = evaluateGTRGAMMAPROTMULTINVAR;
	      evaluateVector    = evaluateGTRGAMMAPROTMULTINVAR_VECTOR;
	      evaluatePartition = evaluateGTRGAMMAPROTMULTPARTITIONINVAR;	 
	      categorizeGAMMA   = categorizeGTRGAMMAPROTMULTINVAR;
	      
	    }
	  else
	    {
	      makenewz          = makenewzGTRGAMMAPROTMULT;
	      makenewzPartition = makenewzGTRGAMMAPROTMULTPARTITION;
	      evaluate          = evaluateGTRGAMMAPROTMULT;
	      evaluateVector    = evaluateGTRGAMMAPROTMULT_VECTOR;
	      evaluatePartition = evaluateGTRGAMMAPROTMULTPARTITION;	 
	      categorizeGAMMA   = categorizeGTRGAMMAPROTMULT;
	    }
	}
      else
	{
	  newview = newviewGTRGAMMAPROT;

	   if(adef->useInvariant)
	     {
	       makenewz        = makenewzGTRGAMMAPROTINVAR;
	       evaluate        = evaluateGTRGAMMAPROTINVAR;	  
	       evaluateVector  = evaluateGTRGAMMAPROTINVAR_VECTOR;
	       categorizeGAMMA = categorizeGTRGAMMAPROTINVAR;
	     }
	   else
	     {
	       makenewz        = makenewzGTRGAMMAPROT;
	       evaluate        = evaluateGTRGAMMAPROT;	  	 	 
	       evaluateVector  = evaluateGTRGAMMAPROT_VECTOR;
	       categorizeGAMMA = categorizeGTRGAMMAPROT;
	     }
	}           
       break;           
    case M_PROTCAT:   
      if(adef->useMultipleModel)
	{
	  newview = newviewGTRCATPROTMULT; 
	  makenewz = makenewzGTRCATPROTMULT; 
	  evaluate = evaluateGTRCATPROTMULT;	 
	  
	  evaluatePartial = evaluatePartialGTRCATPROTMULT;
	}
      else
	{
	  newview = newviewGTRCATPROT;
	  makenewz = makenewzGTRCATPROT;
	  evaluate = evaluateGTRCATPROT;
      
	  evaluatePartial = evaluatePartialGTRCATPROT;
	}            
      break; 
    case M_GTRGAMMA:
      if(adef->useMultipleModel)
	{
	  newviewPartition = newviewGTRGAMMAMULTPARTITION;
	  newview = newviewGTRGAMMAMULT;

	  if(adef->useInvariant)
	    {
	      makenewz          = makenewzGTRGAMMAMULTINVAR;
	      makenewzPartition = makenewzGTRGAMMAMULTINVARPARTITION;
	      evaluate          = evaluateGTRGAMMAMULTINVAR;
	      evaluateVector    = evaluateGTRGAMMAMULTINVAR_VECTOR;
	      evaluatePartition = evaluateGTRGAMMAMULTPARTITIONINVAR;
	      categorizeGAMMA   = categorizeGTRGAMMAMULTINVAR;
	    }
	  else
	    {
	      makenewz          = makenewzGTRGAMMAMULT;
	      makenewzPartition = makenewzGTRGAMMAMULTPARTITION;
	      evaluate          = evaluateGTRGAMMAMULT;
	      evaluateVector    = evaluateGTRGAMMAMULT_VECTOR;
	      evaluatePartition = evaluateGTRGAMMAMULTPARTITION;
	      categorizeGAMMA   = categorizeGTRGAMMAMULT;
	    }	 	  
	}
      else
	{		  
	  newview  = newviewGTRGAMMA;	

	  if(adef->useInvariant)
	    {
	      makenewz        = makenewzGTRGAMMAINVAR;
	      evaluate        = evaluateGTRGAMMAINVAR;
	      evaluateVector  = evaluateGTRGAMMAINVAR_VECTOR;
	      categorizeGAMMA = categorizeGTRGAMMAINVAR;
	    }
	  else
	    {
	      evaluate        = evaluateGTRGAMMA;	  	 		
	      evaluateVector  = evaluateGTRGAMMA_VECTOR;
	      makenewz        = makenewzGTRGAMMA;
	      categorizeGAMMA = categorizeGTRGAMMA;
	    }	  	
	}        
      break;
    case M_GTRCAT:
      if(adef->useMultipleModel)
	{	
	  newview           = newviewGTRCATMULT; 
	  makenewz          = makenewzGTRCATMULT; 
	  evaluate          = evaluateGTRCATMULT;

	  evaluatePartial   = evaluatePartialGTRCATMULT;
	  
	  evaluatePartition = evaluateGTRCATMULTPARTITION;
	  newviewPartition  = newviewPARTITIONGTRCATMULT;
	}
      else
	{      
	  newview         = newviewGTRCAT; 

	  /*
	    
	  Computational experiments on real data have shown that CAT+I does not 
	  give better or worse results than CAT when scored under GTRGAMMA+I,
	  thus it it not worth implementing it        

	  if(adef->useInvariant)
	    {
	      makenewz = makenewzGTRCATINVAR;
	      evaluate = evaluateGTRCATINVAR;
	      evaluatePartial = evaluatePartialGTRCATINVAR;
	      }
	  */
	  
	  makenewz        = makenewzGTRCAT;
	  evaluate        = evaluateGTRCAT;
	  evaluatePartial = evaluatePartialGTRCAT;
	 
	}      
      break;    
    default:
      printf("FATAEL ERROR: assignLikelihoodFunctions\n");
      exit(1);
      break;
    }
}

void initModel(tree *tr, rawdata *rdta, cruncheddata *cdta, analdef *adef)
{  
  int model, i, j;
  double  temp, wtemp;

  if(!adef->useMultipleModel)
    tr->NumberOfModels = 1;
  
  optimizeRatesInvocations = 1;  
  optimizeRateCategoryInvocations = 1;  
  optimizeAlphaInvocations = 1;   
  optimizeInvarInvocations = 1;
  /*optimizeTTRatioInvocations = 1;*/
    

  for(model = 0; model < tr->NumberOfModels; model++)
    {    
      for(i = 0; i < tr->numberOfRates; i++)	    
	tr->initialRates[model * tr->numberOfRates  + i] = 0.5;	       	    
      if(adef->useInvariant)
	{
	  int lower, upper;
	  int count = 0;
	  int total = 0;
	  
	  if(tr->NumberOfModels == 1)
	    {
	      lower = 0;
	      upper = tr->cdta->endsite;
	    }
	  else
	    {
	      lower = tr->modelIndices[model][0];
	      upper = tr->modelIndices[model][1];
	    }
	  
	  for(i = lower; i < upper; i++)
	    {
	      if(tr->invariant[i] < 4) 		
		count += tr->cdta->aliaswgt[i];		  		
	      total += tr->cdta->aliaswgt[i];
	    }
	  tr->invariants[model] = ((double)count)/((double) total);
	  /*printf("Model[%d] Invar: %f\n", model, tr->invariants[model]);*/
	}   
    }

  assignLikelihoodFunctions(adef);

  switch(adef->model)
    {       
    case M_PROTGAMMA:                  
      for(j = 0; j < tr->cdta->endsite; j++)
	tr->cdta->wr[j] = tr->cdta->aliaswgt[j];
      
      baseFrequenciesGTR(rdta, cdta, tr, adef);                 

      for(model = 0; model < tr->NumberOfModels; model++)
	{	
	  tr->alphas[model] = 1.0;
	  initReversibleGTR(tr, adef, model);     
	  makeGammaCats(tr, model);      
	}    
      break;           
    case M_PROTCAT:               
      for(j = 0; j < tr->cdta->endsite; j++)
	tr->cdta->wr[j] = tr->cdta->aliaswgt[j];
      for (j = 0; j < tr->cdta->endsite; j++) 
	{
	  tr->cdta->patrat[j] = temp = 1.0;
	  tr->cdta->patratStored[j] = 1.0;
	  tr->cdta->rateCategory[j] = 0;
	  tr->cdta->wr[j]  = wtemp = temp * tr->cdta->aliaswgt[j];
	  tr->cdta->wr2[j] = temp * wtemp;
	}     
      
      baseFrequenciesGTR(rdta, cdta, tr, adef);

      tr->NumberOfCategories = 1;   
      for(model = 0; model < tr->NumberOfModels; model++)	
	{	 
	  tr->alphas[model] = 1.0; /* GAMMA-APPROX */
	  makeGammaCats(tr, model); /*GAMMA-APPROX */	  
	  initReversibleGTR(tr, adef, model);	  
	}		         
      break; 
    case M_GTRGAMMA:      
      for(j = 0; j < tr->cdta->endsite; j++)
	tr->cdta->wr[j] = tr->cdta->aliaswgt[j];
      baseFrequenciesGTR(rdta, cdta, tr, adef);                 
      for(model = 0; model < tr->NumberOfModels; model++)
	{	
	  tr->alphas[model] = 1.0;
	  initReversibleGTR(tr, adef, model);     
	  makeGammaCats(tr, model);      
	}      
      break;
    case M_GTRCAT:      
      for (j = 0; j < tr->cdta->endsite; j++) 
	{	
	  tr->cdta->patrat[j] = temp = 1.0;
	  tr->cdta->patratStored[j] = 1.0;
	  tr->cdta->rateCategory[j] = 0;
	  tr->cdta->wr[j]  = wtemp = temp * tr->cdta->aliaswgt[j];
	  tr->cdta->wr2[j] = temp * wtemp;
	}     
     
      baseFrequenciesGTR(rdta, cdta, tr,adef);           
      tr->NumberOfCategories = 1;    
      for(model = 0; model < tr->NumberOfModels; model++)
	{	  
	  tr->alphas[model] = 1.0; /* GAMMA-APPROX */
	  makeGammaCats(tr, model); /*GAMMA-APPROX */
	  initReversibleGTR(tr, adef, model);            
	}
      break;    
    }

  if(tr->NumberOfModels > 1)
    {
      tr->fracchange = 0;
      for(model = 0; model < tr->NumberOfModels; model++)
	{
	  tr->fracchange += tr->fracchanges[model];
	}
      tr->fracchange /= ((double)tr->NumberOfModels);
    }

}

